using System;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using LitJson;
using Terraria;
using TAPI;

namespace Bluemagic {
public class Bluemagic : ModBase
{
    public const bool testing = true;
    public static Bluemagic instance;

    public static int helpText = 0;
    private static Color pureColor = new Color(100, 255, 100);
    private static int pureColorStyle = 0;

    public static bool downedPhantom = false;
    public static bool eclipsePassed = false;
    public static bool pumpkinMoonPassed = false;
    public static bool snowMoonPassed = false;
    public static int downedAbomination = 0;

    public override void OnLoad()
    {
        instance = this;
        if(!Main.dedServ)
        {
            BluemagicDust.Load(this);
        }
        AddImmunities();
        SetItemAlphas();
        SetPvpBuffs();
        BluemagicRecipes.AddRecipes();
        SetBiomes();
    }

    private void SetItemAlphas()
    {
        ItemDef.byName["Bluemagic:Icicle"].alpha = 30;
        ItemDef.byName["Bluemagic:PhantomPlate"].alpha = 70;
        ItemDef.byName["Bluemagic:Bubble"].alpha = 30;
        ItemDef.byName["Bluemagic:SuspiciousGel"].alpha = 75;
        ItemDef.byName["Bluemagic:GhoulStaff"].alpha = 70;
    }

    private void AddImmunities()
    {
        int ethereal = BuffDef.byName["Bluemagic:EtherealFlames"];
        NPC dungeonSpirit = NPCDef.byType[288];
        if(dungeonSpirit.buffImmune.Length <= ethereal)
        {
            Array.Resize(ref dungeonSpirit.buffImmune, ethereal + 1);
        }
        NPCDef.byType[288].buffImmune[ethereal] = true;
    }

    private void SetPvpBuffs()
    {
        SetPvpBuff("EtherealFlames");
        SetPvpBuff("Shadow");
        SetPvpBuff("Puppet");
        SetPvpBuff("DarkFlames");
        SetPvpBuff("Sick");
    }

    private void SetPvpBuff(string name)
    {
        int buff = BuffDef.byName["Bluemagic:" + name];
        if(Main.pvpBuff.Length <= buff)
        {
            Array.Resize(ref Main.pvpBuff, buff + 1);
        }
        Main.pvpBuff[buff] = true;
    }

    private void SetBiomes()
    {
        Biome.Biomes["Glowshroom"].typesIncrease.Add(TileDef.byName["Bluemagic:Shroomstone"]);
        Biome.Biomes["Glowshroom"].typesIncrease.Add(TileDef.byName["Bluemagic:Shroomsand"]);
        Biome.Biomes["Desert"].typesIncrease.Add(TileDef.byName["Bluemagic:Shroomsand"]);
        Biome.Biomes["Glowshroom"].typesIncrease.Add(TileDef.byName["Bluemagic:DarkBlueIce"]);
        Biome.Biomes["Snow"].typesIncrease.Add(TileDef.byName["Bluemagic:DarkBlueIce"]);
    }

    public override void PreGameUpdate()
    {
        pureColor.R = (byte)(255 - 155f * Math.Abs(Math.Cos(pureColorStyle * Math.PI / 200.0)));
        pureColor.B = pureColor.R;
        pureColorStyle = (pureColorStyle + 1) % 200;
    }

    public static Color PureColor
    {
        get
        {
            return pureColor;
        }
    }

    public static int ItemID(string name)
    {
        return ItemDef.byName["Bluemagic:" + name].type;
    }

    public static int NPCID(string name)
    {
        return NPCDef.byName["Bluemagic:" + name].type;
    }

    public static int ProjIDToIndex(int id)
    {
        for(int x = 0; x < 1000; x++)
        {
            if(Main.projectile[x].identity == id)
            {
                return x;
            }
        }
        return -1;
    }

    public static bool IsBiomeBlock(string biome, int id)
    {
        return Biome.Biomes[biome].typesIncrease.Contains(id);
    }

    public static bool NoInvasion(int y)
    {
        return !NPC.areaInvasion && ((!Main.pumpkinMoon && !Main.snowMoon) || y > Main.worldSurface || Main.dayTime) && (!Main.eclipse || y > Main.worldSurface || !Main.dayTime);
    }

    public static bool NoBiome(Player spawnedOn, int tile)
    {
        return !Bluemagic.IsBiomeBlock("Jungle", tile) && !spawnedOn.zoneDungeon && !Bluemagic.IsBiomeBlock("Corruption", tile) && !Bluemagic.IsBiomeBlock("Crimson", tile) && !Bluemagic.IsBiomeBlock("Hallow", tile) && !Bluemagic.IsBiomeBlock("Snow", tile) && !Bluemagic.IsBiomeBlock("Glowshroom", tile);
    }

    public static bool NoZoneAllowWater(Player spawnedOn, int tile, int wall)
    {
        return !NPC.areaSky && !spawnedOn.zoneMeteor && !IsBiomeBlock("Meteor", tile) && !NPC.areaNest && wall != 62;
    }

    public static bool NoZone(Player spawnedOn, int tile, int wall)
    {
        return NoZoneAllowWater(spawnedOn, tile, wall) && !NPC.areaWater;
    }

    public static bool NormalSpawn(int y)
    {
        return !NPC.areaSafeTown && NoInvasion(y);
    }

    public static bool NoZoneNormalSpawn(Player spawnedOn, int tile, int wall, int y)
    {
        return NormalSpawn(y) && NoZone(spawnedOn, tile, wall);
    }

    public static bool NoZoneNormalSpawnAllowWater(Player spawnedOn, int tile, int wall, int y)
    {
        return NormalSpawn(y) && NoZoneAllowWater(spawnedOn, tile, wall);
    }

    public static bool NoBiomeNormalSpawn(Player spawnedOn, int tile, int wall, int y)
    {
        return NormalSpawn(y) && NoBiome(spawnedOn, tile) && NoZone(spawnedOn, tile, wall);
    }

    public static void jsonFloat(JsonData data, string name, ref float var)
    {
        if(data.Has(name))
        {
            var = (float)data[name];
        }
    }
}}