using System;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Terraria;
using TAPI;

namespace Bluemagic {
public class BluemagicDust
{
    private static Texture2D texture;

    public static void Load(ModBase modBase)
    {
        texture = modBase.textures["Dust"];
    }

    public static int CreateSpectreDust(float x, float y)
    {
        Color color = new Color(200, 220, 230);
        int dust = Dust.NewDust(new Vector2(x, y), 0, 0, 0, 0f, 0f, 0, color, 1.2f);
        Main.dust[dust].noGravity = true;
        Main.dust[dust].velocity /= 2;
        Main.dust[dust].alpha = 100;
        Main.dust[dust].OverrideTexture = texture;
        Main.dust[dust].OverrideLightColor = color;
        Main.dust[dust].OverrideUpdate = UpdateSpectreDust;
        return dust;
    }

    private static void UpdateSpectreDust(Dust dust)
    {
        dust.position += dust.velocity;
        dust.rotation += dust.velocity.X;
        Lighting.AddLight((int)(dust.position.X / 16f), (int)(dust.position.Y / 16f), 0.05f, 0.15f, 0.2f);
        dust.scale -= 0.03f;
        if(dust.scale < 0.5f)
        {
            dust.active = false;
        }
    }

    public static int CreatePhantomDust(float x, float y)
    {
        int dust = Dust.NewDust(new Vector2(x, y), 0, 0, 1, 0f, 0f, 30, Color.White, 1f);
        Main.dust[dust].noGravity = true;
        Main.dust[dust].OverrideTexture = texture;
        Main.dust[dust].OverrideLightColor = Color.White;
        Main.dust[dust].OverrideUpdate = UpdatePhantomDust;
        return dust;
    }

    private static void UpdatePhantomDust(Dust dust)
    {
        dust.position += dust.velocity;
        dust.rotation += dust.velocity.X;
        Lighting.AddLight((int)(dust.position.X / 16f), (int)(dust.position.Y / 16f), 0.1f, 0.3f, 0.4f);
        dust.alpha += 10;
        if(dust.alpha >= 255)
        {
            dust.alpha = 255;
            dust.active = false;
        }
    }

    public static int CreateEFlameDust(float x, float y, int w, int h, Vector2 velocity = default(Vector2), int direction = 0)
    {
        Color color = new Color(100, 200, 255);
        int dust = Dust.NewDust(new Vector2(x, y), w, h, 0, velocity.X * 0.2f + 3 * direction, velocity.Y * 0.2f, 0, color, 2.5f);
        Main.dust[dust].noGravity = true;
        Main.dust[dust].alpha = 100;
        Main.dust[dust].velocity *= 0.5f;
        Main.dust[dust].OverrideTexture = texture;
        Main.dust[dust].OverrideLightColor = color;
        Main.dust[dust].OverrideUpdate = UpdateEFlameDust;
        return dust;
    }

    private static void UpdateEFlameDust(Dust dust)
    {
        dust.position += dust.velocity;
        dust.rotation += dust.velocity.X;
        Lighting.AddLight((int)(dust.position.X / 16f), (int)(dust.position.Y / 16f), 0.05f, 0.15f, 0.2f);
        dust.scale -= 0.1f;
        if(dust.scale < 0.5f)
        {
            dust.active = false;
        }
    }

    public static int CreateSparkle(float x, float y)
    {
        return CreateSparkle(new Vector2(x, y), 0, 0);
    }

    public static int CreateSparkle(Vector2 position, int width, int height)
    {
        int dust = Dust.NewDust(position, width, height, 2, 0f, 0f, 0, Color.White, 2f);
        Main.dust[dust].noGravity = true;
        Main.dust[dust].OverrideTexture = texture;
        Main.dust[dust].OverrideLightColor = Color.White;
        Main.dust[dust].OverrideUpdate = UpdateSparkle;
        return dust;
    }

    private static void UpdateSparkle(Dust dust)
    {
        dust.position += dust.velocity;
        dust.rotation += 0.05f;
        dust.scale -= 0.1f;
        if(dust.scale < 0.5f)
        {
            dust.active = false;
        }
    }

    public static int CreateFlame(Vector2 pos, int width, int height, Color color)
    {
        int dust = Dust.NewDust(pos, width, height, 3, 0f, 0f, 0, color, 2f);
        Main.dust[dust].noGravity = true;
        Main.dust[dust].OverrideTexture = texture;
        Main.dust[dust].OverrideLightColor = color;
        Main.dust[dust].OverrideUpdate = UpdateFlame;
        return dust;
    }

    private static void UpdateFlame(Dust dust)
    {
        dust.position += dust.velocity;
        dust.rotation += dust.velocity.X;
        dust.scale -= 0.1f;
        dust.velocity *= 0.95f;
        if(dust.scale < 0.5f)
        {
            dust.active = false;
        }
        else
        {
            float strength = dust.scale / 2f;
            Lighting.AddLight((int)(dust.position.X / 16f), (int)(dust.position.Y / 16f), dust.color.R / 255f * 0.5f * strength, dust.color.G / 255f * 0.5f * strength, dust.color.B / 255f * 0.4f * strength);
        }
    }

    public static int CreatePixel(Vector2 pos, int width, int height, float rotation, Color color, bool adjust = true)
    {
        int dust = Dust.NewDust(pos, width, height, 4, 0f, 0f, 0, color, 1f);
        if(adjust)
        {
            Vector2 unit = new Vector2((float)Math.Cos(rotation), (float)Math.Sin(rotation));
            Vector2 center = pos + (new Vector2((float)width / 2f, (float)height / 2f));
            Vector2 offset = Main.dust[dust].position - center;
            offset.X = (offset.X - (float)width / 2f) / 2f;
            Main.dust[dust].position = center + new Vector2(unit.X * offset.X - unit.Y * offset.Y, unit.Y * offset.X + unit.X * offset.Y);
            Main.dust[dust].velocity += -3f * unit;
        }
        Main.dust[dust].noGravity = true;
        Main.dust[dust].rotation = rotation;
        Main.dust[dust].OverrideTexture = texture;
        Main.dust[dust].OverrideLightColor = color;
        Main.dust[dust].OverrideUpdate = UpdatePixel;
        return dust;
    }

    private static void UpdatePixel(Dust dust)
    {
        dust.position += dust.velocity;
        dust.scale -= 0.01f;
        if(dust.scale < 0.75f)
        {
            dust.active = false;
        }
        else
        {
            Lighting.AddLight((int)(dust.position.X / 16f), (int)(dust.position.Y / 16f), dust.color.R / 255f * 0.5f, dust.color.G / 255f * 0.5f, dust.color.B / 255f * 0.4f);
        }
    }

    public static int CreatePixelHurt(Vector2 pos, int width, int height, Color color)
    {
        int dust = Dust.NewDust(pos, width, height, 4, 0f, 0f, 0, color, 1f);
        Main.dust[dust].noGravity = true;
        Main.dust[dust].OverrideTexture = texture;
        Main.dust[dust].OverrideLightColor = color;
        Main.dust[dust].OverrideUpdate = UpdatePixel;
        return dust;
    }

    private static void UpdatePixelHurt(Dust dust)
    {
        dust.position += dust.velocity;
        dust.velocity *= 0.97f;
        if(Math.Sqrt(dust.velocity.X * dust.velocity.X + dust.velocity.Y * dust.velocity.Y) < 0.001)
        {
            dust.active = false;
        }
    }

    public static int CreatePuriumBulletDust(Vector2 pos)
    {
        int dust = Dust.NewDust(pos, 0, 0, 5, 0f, 0f, 0, Color.White, 1f);
        Main.dust[dust].noGravity = true;
        Main.dust[dust].velocity = Vector2.Zero;
        Main.dust[dust].OverrideTexture = texture;
        Main.dust[dust].OverrideLightColor = Color.White;
        Main.dust[dust].OverrideUpdate = UpdatePuriumBulletDust;
        return dust;
    }

    private static void UpdatePuriumBulletDust(Dust dust)
    {
        dust.alpha += 20;
        dust.scale -= 0.04f;
        if(dust.alpha >= 255)
        {
            dust.alpha = 255;
            dust.active = false;
        }
    }

    public static int CreateRuneDust(Vector2 pos, int width, int height)
    {
        int flag = Main.rand.Next(6);
        Color color = Color.Black;
        if(flag < 2 || flag == 5)
        {
            color.R = 255;
        }
        if(flag >= 1 && flag <= 3)
        {
            color.G = 255;
        }
        if(flag >= 3)
        {
            color.B = 255;
        }
        int dust = Dust.NewDust(pos, width, height, 6, 0f, 0f, 0, color, 2f);
        Main.dust[dust].noGravity = true;
        Main.dust[dust].OverrideTexture = texture;
        Main.dust[dust].OverrideLightColor = color;
        Main.dust[dust].OverrideUpdate = UpdateRuneDust;
        return dust;
    }

    private static void UpdateRuneDust(Dust dust)
    {
        dust.position += dust.velocity;
        dust.scale -= 0.02f;
        dust.rotation += dust.velocity.X / 2f;
        if(dust.scale <= 1f)
        {
            dust.active = false;
        }
    }

    public static int CreateSparkleAura(Vector2 pos, int width, int height, Color color)
    {
        int dust = Dust.NewDust(pos, width, height, 7, 0f, 0f, 0, color, 1f);
        Main.dust[dust].noGravity = true;
        Main.dust[dust].velocity *= 0.1f;
        Main.dust[dust].velocity.Y -= 0.1f;
        Main.dust[dust].OverrideTexture = texture;
        Main.dust[dust].OverrideLightColor = color;
        Main.dust[dust].OverrideUpdate = UpdateSparkleAura;
        return dust;
    }

    private static void UpdateSparkleAura(Dust dust)
    {
        dust.position += dust.velocity;
        dust.alpha += 5;
        dust.rotation += dust.velocity.X / 3f;
        if(dust.alpha >= 255)
        {
            dust.alpha = 255;
            dust.active = false;
        }
    }

    public static int CreatePuriumSliceDust(Vector2 pos, int width, int height)
    {
        Color color = new Color(100, 255, 120);
        if(Main.rand.Next(3) == 0)
        {
            color = new Color(200, 255, 120);
        }
        int dust = Dust.NewDust(pos, width, height, 0, 0f, 0f, 100, color, 1f);
        Main.dust[dust].OverrideTexture = texture;
        Main.dust[dust].OverrideUpdate = UpdatePuriumSliceDust;
        return dust;
    }

    private static void UpdatePuriumSliceDust(Dust dust)
    {
        if(!dust.noGravity)
        {
            dust.velocity.Y += 0.5f;
        }
        dust.velocity.X *= 0.98f;
        dust.position += dust.velocity;
        Lighting.AddLight((int)(dust.position.X / 16f), (int)(dust.position.Y / 16f), 0.3f, 0.6f, 0.2f);
        dust.alpha += 5;
        if(dust.alpha >= 150)
        {
            dust.active = false;
        }
    }

    public static int CreatePuriumFlameDust(Vector2 pos, Vector2 vel, int width, int height)
    {
        int dust = Dust.NewDust(pos, width, height, 0, 0f, 0f, 1, new Color(170, 240, 140), 1.1f);
        Main.dust[dust].velocity *= 0.2f;
        Main.dust[dust].velocity += vel;
        Main.dust[dust].noGravity = true;
        Main.dust[dust].OverrideTexture = texture;
        Main.dust[dust].OverrideLightColor = new Color(140, 250, 100);
        Main.dust[dust].OverrideUpdate = UpdatePuriumFlameDust;
        return dust;
    }

    private static void UpdatePuriumFlameDust(Dust dust)
    {
        dust.rotation += dust.velocity.X / 3f;
        dust.position += dust.velocity;
        int oldAlpha = dust.alpha;
        dust.alpha = (int)(dust.alpha * 1.2);
        if(dust.alpha == oldAlpha)
        {
            dust.alpha++;
        }
        if(dust.alpha >= 255)
        {
            dust.alpha = 255;
            dust.active = false;
        }
    }

    public static int CreateSmoke(Vector2 pos, int width, int height, Color color)
    {
        int dust = Dust.NewDust(pos, width, height, 8, 0f, 0f, 0, color, 1f);
        Main.dust[dust].noGravity = true;
        Main.dust[dust].velocity *= 0.5f;
        Main.dust[dust].velocity.Y -= 0.5f;
        Main.dust[dust].OverrideTexture = texture;
        Main.dust[dust].OverrideUpdate = UpdateSmoke;
        return dust;
    }

    private static void UpdateSmoke(Dust dust)
    {
        dust.position += dust.velocity;
        dust.rotation += dust.velocity.X * 0.1f;
        dust.scale -= 0.02f;
        if(dust.scale < 0.5f)
        {
            dust.active = false;
        }
    }

    public static int CreateSpark(Vector2 pos, int width, int height)
    {
        int dust = Dust.NewDust(pos, width, height, 9, 0f, 0f, 0, Color.White, 1f);
        Main.dust[dust].velocity.Y -= 1f;
        Main.dust[dust].OverrideTexture = texture;
        Main.dust[dust].OverrideUpdate = UpdateSpark;
        return dust;
    }

    private static void UpdateSpark(Dust dust)
    {
        dust.position += dust.velocity;
        dust.velocity.Y += 0.05f;
        dust.scale -= 0.01f;
        float light = dust.scale * (dust.frame.Y == 10f ? 0.5f : 1f);
        Lighting.AddLight((int)(dust.position.X / 16f), (int)(dust.position.Y / 16f), 0.7f * light, 0.7f * light, 0.4f * light);
        dust.rotation += dust.velocity.X / 10f;
        if(dust.scale < 0.5f)
        {
            dust.active = false;
        }
    }

    public static int CreateDFlameDust(float x, float y, int w, int h, Vector2 velocity = default(Vector2), int direction = 0)
    {
        Color color = new Color(50, 5, 60);
        int dust = Dust.NewDust(new Vector2(x, y), w, h, 0, velocity.X * 0.2f + 3 * direction, velocity.Y * 0.2f, 0, color, 2.5f);
        Main.dust[dust].noGravity = true;
        Main.dust[dust].alpha = 100;
        Main.dust[dust].velocity *= 0.5f;
        Main.dust[dust].OverrideTexture = texture;
        Main.dust[dust].OverrideLightColor = color;
        Main.dust[dust].OverrideUpdate = UpdateDFlameDust;
        return dust;
    }

    private static void UpdateDFlameDust(Dust dust)
    {
        dust.position += dust.velocity;
        dust.rotation += dust.velocity.X;
        dust.scale -= 0.1f;
        if(dust.scale < 0.5f)
        {
            dust.active = false;
        }
    }
}}