using System;
using Microsoft.Xna.Framework;
using Terraria;
using TAPI;

using Bluemagic.NPCs;
using Bluemagic.Projectiles;

namespace Bluemagic {
public class BluemagicNet : ModNet
{
    public static void SendNPCData(int npc, int ignore = -1)
    {
        NetMessage.SendData(23, -1, ignore, "", npc, 0f, 0f, 0f, 0f);
    }

    public static void NewText(string text, byte r, byte g, byte b)
    {
        if(Main.netMode == 0)
        {
            Main.NewText(text, r, g, b, false);
        }
        else if(Main.netMode == 2)
        {
            NetMessage.SendData(25, -1, -1, text, 255, (float)r, (float)g, (float)b, 0f);
        }
    }

    public static void NewClientText(string text, byte r, byte g, byte b)
    {
        if(Main.netMode != 2)
        {
            Main.NewText(text, r, g, b, false);
        }
    }

    public static void SendNPCLocalData(int npc, int ignore = -1)
    {
        float[] localAI = Main.npc[npc].localAI;
        NetMessage.SendModData(Bluemagic.instance, 1, -1, ignore, npc, localAI[0], localAI[1], localAI[2], localAI[3]);
    }

    public static void SendNPCAllData(int npc, int ignore = -1)
    {
        SendNPCData(npc, ignore);
        SendNPCLocalData(npc, ignore);
    }

    public static bool SendProjData(int proj, int ignore = -1)
    {
        if(Main.projectile[proj].owner == Main.myPlayer)
        {
            NetMessage.SendData(27, -1, ignore, "", proj, 0f, 0f, 0f, 0f);
            return true;
        }
        return false;
    }

    public static bool SendProjLocalData(int proj, int ignore = -1)
    {
        if(Main.projectile[proj].owner == Main.myPlayer)
        {
            float[] localAI = Main.projectile[proj].localAI;
            NetMessage.SendModData(Bluemagic.instance, 2, -1, ignore, Main.projectile[proj].identity, localAI[0], localAI[1]);
            return true;
        }
        return false;
    }

    public static bool SendProjAllData(int proj, int ignore = -1)
    {
        SendProjData(proj, ignore);
        return SendProjLocalData(proj, ignore);
    }

    public static void SendProjKill(int proj, int ignore = -1)
    {
        Projectile projectile = Main.projectile[proj];
        NetMessage.SendData(29, -1, -1, "", projectile.identity, (float)projectile.owner, 0f, 0f, 0f);
    }

    public static void SendNPCHeal(int npc, int amount)
    {
        if(Main.netMode != 2)
        {
            NPC npc0 = Main.npc[npc];
            CombatText.NewText(new Rectangle((int)npc0.position.X, (int)npc0.position.Y, npc0.width, npc0.height), new Color(100, 255, 100, 255), string.Concat(amount), false, false);
        }
        if(Main.netMode != 0)
        {
            NetMessage.SendModData(Bluemagic.instance, 3, -1, -1, npc, amount);
        }
    }

    public static void SendSound(int type, int x, int y, int style)
    {
        NetMessage.SendModData(Bluemagic.instance, 4, -1, -1, type, x, y, style);
    }

    public static void PlaySound(int type, int x, int y, int style)
    {
        if(Main.netMode != 2)
        {
            Main.PlaySound(type, x, y, style);
        }
        if(Main.netMode != 0)
        {
            SendSound(type, x, y, style);
        }
    }

    public static void SendTransform(int player, int type, int time, int ignore = -1)
    {
        NetMessage.SendModData(Bluemagic.instance, 5, -1, ignore, player, type, time);
    }

    public static void QueryTransform(int player)
    {
        if(Main.netMode == 2)
        {
            NetMessage.SendModData(Bluemagic.instance, 6, -1, -1, player);
        }
    }

    public static void SendProjMaxUpdates(int proj, int ignore = -1)
    {
        NetMessage.SendModData(Bluemagic.instance, 7, -1, ignore, Main.projectile[proj].identity, Main.projectile[proj].maxUpdates);
    }

    public static void SendTrailAdd(int proj, Vector2 pos, int ignore = -1)
    {
        NetMessage.SendModData(Bluemagic.instance, 8, -1, ignore, 0, Main.projectile[proj].identity, pos.X, pos.Y);
    }

    public static void SendAbominationData(int npc)
    {
        Abomination boss = Main.npc[npc].GetSubClass<Abomination>();
        NetMessage.SendModData(Bluemagic.instance, 9, -1, -1, npc, boss.moveTime);
    }

    public static void SendTransceiverSwitch(int x, int y, int ignore = -1)
    {
        NetMessage.SendModData(Bluemagic.instance, 10, -1, ignore, x, y);
    }

    public static void SendSandPlacement(int tileID, string conditions, int ignore = -1)
    {
        NetMessage.SendModData(Bluemagic.instance, 11, -1, ignore, tileID, conditions);
    }

    public static void SendShadowArmData(int proj, int ignore = -1)
    {
        Vector2 pos = Main.projectile[proj].GetSubClass<ShadowArm>().target;
        NetMessage.SendModData(Bluemagic.instance, 8, -1, ignore, 1, Main.projectile[proj].identity, pos.X, pos.Y);
    }

    public static void SendOctopusArmData(int proj, int ignore = -1)
    {
        if(SendProjAllData(proj, ignore))
        {
            OctopusArm arm = Main.projectile[proj].GetSubClass<OctopusArm>();
            NetMessage.SendModData(Bluemagic.instance, 12, -1, ignore, 0, Main.projectile[proj].identity, arm.minAngle, arm.maxAngle, arm.angleSpeed, arm.lengthSpeed);
        }
    }

    public static void SendNightmareDropData(int proj, int amount, int ignore = -1)
    {
        NetMessage.SendModData(Bluemagic.instance, 12, -1, ignore, 1, Main.projectile[proj].identity, amount);
    }

    public override void NetReceive(BinBuffer bb, int messageID, MessageBuffer buffer)
    {
        switch(messageID)
        {
        case 1:
        {
            int npc = bb.ReadInt();
            float[] localAI = Main.npc[npc].localAI;
            localAI[0] = bb.ReadFloat();
            localAI[1] = bb.ReadFloat();
            localAI[2] = bb.ReadFloat();
            localAI[3] = bb.ReadFloat();
            if(Main.netMode == 2)
            {
                SendNPCLocalData(npc, buffer.whoAmI);
            }
            break;
        }
        case 2:
        {
            int proj = Bluemagic.ProjIDToIndex(bb.ReadInt());
            float[] localAI = new float[2];
            localAI[0] = bb.ReadFloat();
            localAI[1] = bb.ReadFloat();
            if(Main.netMode == 2)
            {
                NetMessage.SendModData(Mods.GetMod("Bluemagic"), 2, -1, buffer.whoAmI, proj, localAI[0], localAI[1]);
            }
            Main.projectile[proj].localAI[0] = localAI[0];
            Main.projectile[proj].localAI[1] = localAI[1];
            break;
        }
        case 3:
        {
            NPC npc = Main.npc[bb.ReadInt()];
            int amount = bb.ReadInt();
            CombatText.NewText(new Rectangle((int)npc.position.X, (int)npc.position.Y, npc.width, npc.height), new Color(100, 255, 100, 255), string.Concat(amount), false, false);
            npc.life += amount;
            if(npc.life > npc.lifeMax)
            {
                npc.life = npc.lifeMax;
            }
            if(Main.netMode == 2)
            {
                NetMessage.SendModData(Mods.GetMod("Bluemagic"), 3, -1, buffer.whoAmI, npc.whoAmI, amount);
            }
            break;
        }
        case 4:
            if(Main.netMode == 1)
            {
                Main.PlaySound(bb.ReadInt(), bb.ReadInt(), bb.ReadInt(), bb.ReadInt());
            }
            if(Main.netMode == 2)
            {
                NetMessage.SendModData(Mods.GetMod("Bluemagic"), 4, -1, buffer.whoAmI, bb.ReadInt(), bb.ReadInt(), bb.ReadInt(), bb.ReadInt());
            }
            break;
        case 5:
        {
            int player = bb.ReadInt();
            int type = bb.ReadInt();
            int time = bb.ReadInt();
            if(type == 1)
            {
                Main.player[player].AddBuff("Bluemagic:Liquified", time, true);
            }
            else if(type == 2)
            {
                Main.player[player].AddBuff("Bluemagic:Rotting", time, true);
            }
            else if(type == 3)
            {
                Main.player[player].AddBuff("Bluemagic:Fleshy", time, true);
            }
            if(Main.netMode == 2)
            {
                SendTransform(player, type, time, buffer.whoAmI);
            }
            break;
        }
        case 6:
        {
            int player = bb.ReadInt();
            if(Main.netMode == 1 && player == Main.myPlayer)
            {
                for(int k = 0; k < 22; k++)
                {
                    if(Main.player[player].buffType[k] == BuffDef.byName["Bluemagic:Liquified"])
                    {
                        SendTransform(player, 1, Main.player[player].buffTime[k]);
                    }
                    else if(Main.player[player].buffType[k] == BuffDef.byName["Bluemagic:Rotting"])
                    {
                        SendTransform(player, 2, Main.player[player].buffTime[k]);
                    }
                    else if(Main.player[player].buffType[k] == BuffDef.byName["Bluemagic:Fleshy"])
                    {
                        SendTransform(player, 3, Main.player[player].buffTime[k]);
                    }
                }
            }
            break;
        }
        case 7:
        {
            int proj = Bluemagic.ProjIDToIndex(bb.ReadInt());
            Main.projectile[proj].maxUpdates = bb.ReadInt();
            if(Main.netMode == 2)
            {
                SendProjMaxUpdates(proj, buffer.whoAmI);
            }
            break;
        }
        case 8:
        {
            int type = bb.ReadInt();
            int proj = Bluemagic.ProjIDToIndex(bb.ReadInt());
            Vector2 pos = new Vector2(bb.ReadFloat(), bb.ReadFloat());
            if(type == 0)
            {
                TrailHitbox trail = Main.projectile[proj].GetSubClass<TrailHitbox>();
                if(trail != null)
                {
                    trail.Add(pos);
                }
            }
            else if(type == 1)
            {
                ShadowArm arm = Main.projectile[proj].GetSubClass<ShadowArm>();
                if(arm != null)
                {
                    arm.target = pos;
                }
            }
            if(Main.netMode == 2)
            {
                NetMessage.SendModData(Bluemagic.instance, 8, -1, buffer.whoAmI, type, proj, pos.X, pos.Y);
            }
            break;
        }
        case 9:
        {
            int npc = bb.ReadInt();
            int moveTime = bb.ReadInt();
            Abomination boss = Main.npc[npc].GetSubClass<Abomination>();
            if(boss != null)
            {
                boss.moveTime = moveTime;
            }
            break;
        }
        case 10:
        {
            int x = bb.ReadInt();
            int y = bb.ReadInt();
            Tiles.WirelessTransceiver.Switch(x, y);
            if(Main.netMode == 2)
            {
                SendTransceiverSwitch(x, y, buffer.whoAmI);
            }
            break;
        }
        case 11:
        {
            int tileID = bb.ReadInt();
            string conditions = bb.ReadString();
            TileDef.placementConditions[tileID] = conditions;
            if(Main.netMode == 2)
            {
                SendSandPlacement(tileID, conditions, buffer.whoAmI);
            }
            break;
        }
        case 12:
        {
            int type = bb.ReadInt();
            int proj = Bluemagic.ProjIDToIndex(bb.ReadInt());
            if(type == 0)
            {
                float minAngle = bb.ReadFloat();
                float maxAngle = bb.ReadFloat();
                float angleSpeed = bb.ReadFloat();
                float lengthSpeed = bb.ReadFloat();
                OctopusArm arm = Main.projectile[proj].GetSubClass<OctopusArm>();
                if(arm != null)
                {
                    arm.minAngle = minAngle;
                    arm.maxAngle = maxAngle;
                    arm.angleSpeed = angleSpeed;
                    arm.lengthSpeed = lengthSpeed;
                }
            }
            else if(type == 1)
            {
                int amount = bb.ReadInt();
                NightmareDrop drop = Main.projectile[proj].GetSubClass<NightmareDrop>();
                if(drop != null)
                {
                    drop.LeechLife(amount);
                }
                if(Main.netMode == 2)
                {
                    SendNightmareDropData(Main.projectile[proj].whoAmI, amount, buffer.whoAmI);
                }
            }
            break;
        }
        default:
            break;
        }
    }

    public override void PlayerConnected(int index)
    {
        QueryTransform(index);
    }

    public static void Feedback(string message)
    {
        if(Main.netMode == 2)
        {
            NetMessage.SendData(25, -1, -1, "<Server> " + message, 255, 255, 255, 255, 0f);
        }
        else
        {
            Main.NewText("<Client> " + message, 255, 255, 255, false);
        }
    }

    private static void Feedback(bool sending, string message)
    {
        if(sending)
        {
            Feedback(" (sending) " + message);
        }
        else
        {
            Feedback(" (received) " + message);
        }
        
    }
}}