using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Terraria;
using Terraria.DataStructures;
using TAPI;

namespace Bluemagic {
public class BluemagicWorld : ModWorld
{
    private static List<ItemText> updateTextsPure = new List<ItemText>();
    private static List<ItemText> updateTextsRainbow = new List<ItemText>();
    public override void PostUpdate()
    {
        if(Main.eclipse)
        {
            Bluemagic.eclipsePassed = true;
        }
        if(Main.pumpkinMoon && NPC.waveCount >= 15)
        {
            Bluemagic.pumpkinMoonPassed = true;
        }
        if(Main.snowMoon && NPC.waveCount >= 15)
        {
            Bluemagic.snowMoonPassed = true;
        }
        for(int k = 0; k < updateTextsPure.Count; k++)
        {
            ItemText text = updateTextsPure[k];
            if(text.active)
            {
                text.color = Bluemagic.PureColor;
            }
            else
            {
                updateTextsPure.Remove(text);
                k--;
            }
        }
        for(int k = 0; k < updateTextsRainbow.Count; k++)
        {
            ItemText text = updateTextsRainbow[k];
            if(text.active)
            {
                text.color = new Color(Main.DiscoR, Main.DiscoG, Main.DiscoB);
            }
            else
            {
                updateTextsRainbow.Remove(text);
                k--;
            }
        }

        if(Main.netMode != 1)
        {
            for(int k = 0; k < Main.maxTilesX * Main.maxTilesY * 3E-05 * Main.worldRate; k++)
            {
                int x = WorldGen.genRand.Next(10, Main.maxTilesX - 10);
                int y = WorldGen.genRand.Next(10, (int)Main.worldSurface - 1);
                if(Main.tile[x, y] != null && Main.tile[x, y].liquid <= 32 && Main.tile[x, y].nactive())
                {
                    UpdateTile(x, y);
                }
            }
            for(int k = 0; k < Main.maxTilesX * Main.maxTilesY * 1.5E-05 * Main.worldRate; k++)
            {
                int x = WorldGen.genRand.Next(10, Main.maxTilesX - 10);
                int y = WorldGen.genRand.Next((int)Main.worldSurface - 1, Main.maxTilesY - 20);
                if(Main.tile[x, y] != null && Main.tile[x, y].liquid <= 32 && Main.tile[x, y].nactive())
                {
                    UpdateTile(x, y);
                }
            }
        }
    }

    private void UpdateTile(int x, int y)
    {
        Tile tile = Main.tile[x, y];
        if(!tile.inActive() && (tile.type == 70 || tile.type == TileDef.byName["Bluemagic:Shroomstone"] || tile.type == TileDef.byName["Bluemagic:Shroomsand"] || tile.type == TileDef.byName["Bluemagic:DarkBlueIce"]) && Bluemagic.downedAbomination > 0 && WorldGen.genRand.Next(3) == 0)
        {
            bool flag = true;
            while(flag)
            {
                flag = false;
                int toX = x + WorldGen.genRand.Next(-3, 4);
                int toY = y + WorldGen.genRand.Next(-3, 4);
                bool tileChanged = false;
                int targetType = Main.tile[toX, toY].type;
                if(targetType == 1 || TileDef.moss[targetType])
                {
                    Main.tile[toX, toY].type = TileDef.byName["Bluemagic:Shroomstone"];
                    tileChanged = true;
                }
                else if(targetType == 0 && toY > Main.worldSurface)
                {
                    Main.tile[toX, toY].type = 59;
                    tileChanged = true;
                }
                else if(targetType == 59)
                {
                    for(int j = toX - 1; j <= toX + 1; j++)
                    {
                        for(int k = toY - 1; k <= toY + 1; k++)
                        {
                            if(!Main.tile[j, k].active() || !TileDef.solid[Main.tile[j, k].type])
                            {
                                tileChanged = true;
                            }
                            if(Main.tile[j, k].lava() && Main.tile[j, k].liquid > 0)
                            {
                                tileChanged = false;
                                j = toX + 1;
                                break;
                            }
                        }
                    }
                    if(tileChanged)
                    {
                        Main.tile[toX, toY].type = 70;
                    }
                }
                else if(targetType == 60)
                {
                    Main.tile[toX, toY].type = 70;
                    tileChanged = true;
                }
                else if(targetType == 53)
                {
                    Main.tile[toX, toY].type = TileDef.byName["Bluemagic:Shroomsand"];
                    tileChanged = true;
                }
                else if(targetType == 161)
                {
                    Main.tile[toX, toY].type = TileDef.byName["Bluemagic:DarkBlueIce"];
                    tileChanged = true;
                }
                if(tileChanged)
                {
                    if(WorldGen.genRand.Next(2) == 0)
                    {
                        flag = true;
                    }
                    WorldGen.SquareTileFrame(toX, toY, true);
                    NetMessage.SendTileSquare(-1, toX, toY, 1);
                }
            }
        }
    }

    public static void GenPurium()
    {
        if(Main.netMode == 1 || WorldGen.noTileActions || WorldGen.gen)
        {
            return;
        }
        Bluemagic.downedAbomination += 1;
        for(double k = 0; k < (Main.maxTilesX - 200) * (Main.maxTilesY - 150 - (int)Main.rockLayer) / 10000.0 / (double)Bluemagic.downedAbomination; k += 1.0)
        {
            WorldGen.OreRunner(WorldGen.genRand.Next(100, Main.maxTilesX - 100), WorldGen.genRand.Next((int)Main.rockLayer, Main.maxTilesY - 150), (double)WorldGen.genRand.Next(4, 8), WorldGen.genRand.Next(4, 8), TileDef.byName["Bluemagic:PuriumOre"]);
        }
        BluemagicNet.NewText("The elements have been unleashed!", 100, 220, 100);
    }

    public static void AddText(ItemText text, int rare)
    {
        if(rare == 12)
        {
            updateTextsPure.Add(text);
        }
        if(rare >= 13)
        {
            updateTextsRainbow.Add(text);
        }
    }

    public override void Save(BinBuffer bb)
    {
        WriteEntry(bb, "downedPhantom", Bluemagic.downedPhantom);
        WriteEntry(bb, "eclipsePassed", Bluemagic.eclipsePassed);
        WriteEntry(bb, "pumpkinMoonPassed", Bluemagic.pumpkinMoonPassed);
        WriteEntry(bb, "snowMoonPassed", Bluemagic.snowMoonPassed);
        WriteEntry(bb, "downedAbomination", Bluemagic.downedAbomination);
        WriteEntry(bb, "transceivers", Tiles.WirelessTransceiver.positions);

        bb.Write("EOF");
    }

    private void WriteEntry(BinBuffer bb, string tag, bool val)
    {
        bb.Write(tag);
        bb.Write(val);
    }

    private void WriteEntry(BinBuffer bb, string tag, int val)
    {
        bb.Write(tag);
        bb.Write(val);
    }

    private void WriteEntry(BinBuffer bb, string tag, List<Point16> points)
    {
        bb.Write(tag);
        foreach(Point16 point in points)
        {
            bb.Write(point.x);
            bb.Write(point.y);
        }
        bb.Write((short)-1);
    }

    public override void Load(BinBuffer bb)
    {
        WrapperDictionary<string, bool> entriesBool = new WrapperDictionary<string, bool>();
        WrapperDictionary<string, int> entriesInt = new WrapperDictionary<string, int>();
        string tag = bb.ReadString();
        while(!tag.Equals("EOF"))
        {
            if(tag.Equals("downedAbomination"))
            {
                entriesInt[tag] = bb.ReadInt();
            }
            else if(tag.Equals("transceivers"))
            {
                LoadData(bb, Tiles.WirelessTransceiver.positions);
            }
            else
            {
                entriesBool[tag] = bb.ReadBool();
            }
            tag = bb.ReadString();
        }
        LoadData(entriesBool, "downedPhantom", ref Bluemagic.downedPhantom, false);
        LoadData(entriesBool, "eclipsePassed", ref Bluemagic.eclipsePassed, false);
        LoadData(entriesBool, "pumpkinMoonPassed", ref Bluemagic.pumpkinMoonPassed, false);
        LoadData(entriesBool, "snowMoonPassed", ref Bluemagic.snowMoonPassed, false);
        LoadData(entriesInt, "downedAbomination", ref Bluemagic.downedAbomination, 0);
    }

    private void LoadData(WrapperDictionary<string, bool> entries, string tag, ref bool slot, bool default0)
    {
        if(entries.ContainsKey(tag))
        {
            slot = entries[tag];
        }
        else
        {
            slot = default0;
        }
    }

    private void LoadData(WrapperDictionary<string, int> entries, string tag, ref int slot, int default0)
    {
        if(entries.ContainsKey(tag))
        {
            slot = entries[tag];
        }
        else
        {
            slot = default0;
        }
    }

    private void LoadData(BinBuffer bb, List<Point16> points)
    {
        points.Clear();
        short x = bb.ReadShort();
        while(x >= 0)
        {
            short y = bb.ReadShort();
            points.Add(new Point16(x, y));
            x = bb.ReadShort();
        }
    }
}}