using System;
using Microsoft.Xna.Framework;
using Terraria;
using TAPI;

namespace Bluemagic.NPCs {
public class CaptiveElement : ModNPC
{
    //ai[0] = center
    //ai[1] = type
    //ai[2] = attackCool
    //ai[3] = change
    //localAI[0] = setup

    public override void AI()
    {
        NPC abomination = Main.npc[(int)npc.ai[0]];
        if(!abomination.active || abomination.type != NPCDef.byName["Bluemagic:Abomination"].type)
        {
            if(npc.ai[3] == 0f)
            {
                npc.netUpdate = true;
            }
            npc.ai[3] += 1f;
        }
        if(npc.ai[3] > 0f)
        {
            Color? color = GetColor();
            if(color.HasValue)
            {
                for(int x = 0; x < 5; x++)
                {
                    int dust = BluemagicDust.CreatePixel(npc.position, npc.width, npc.height, npc.rotation, color.Value, false);
                    double angle = Main.rand.NextDouble() * 2.0 * Math.PI;
                    Main.dust[dust].velocity = 3f * new Vector2((float)Math.Cos(angle), (float)Math.Sin(angle));
                }
            }
            if(Main.netMode != 1 && npc.ai[3] >= 100f)
            {
                int next = NPC.NewNPC((int)npc.Center.X, (int)npc.position.Y + npc.height, NPCDef.byName["Bluemagic:CaptiveElement2"].type, 0);
                Main.npc[next].ai[0] = npc.ai[1];
                if(npc.ai[1] != 4f)
                {
                    Main.npc[next].ai[1] = 300f + (float)Main.rand.Next(100);
                }
                npc.life = -1;
                npc.active = false;
            }
            return;
        }
        else if(npc.timeLeft < 750)
        {
            npc.timeLeft = 750;
        }
        if(npc.localAI[0] == 0f)
        {
            if(GetDebuff() >= 0f)
            {
                if(GetDebuff() >= npc.buffImmune.Length)
                {
                    Array.Resize<bool>(ref npc.buffImmune, GetDebuff() + 1);
                }
                npc.buffImmune[GetDebuff()] = true;
            }
            if(npc.ai[1] == 3f)
            {
                npc.buffImmune[20] = true;
            }
            if(npc.ai[1] == 0f)
            {
                npc.coldDamage = true;
            }
            npc.localAI[0] = 1f;
        }

        SetPosition(npc);
        npc.ai[2] -= 1f;
        if(Main.netMode != 1 && npc.ai[2] <= 0f)
        {
            npc.ai[2] = 200f + 200f * (float)abomination.life / (float)abomination.lifeMax + (float)Main.rand.Next(200);
            Vector2 delta = Main.player[abomination.target].Center - npc.Center;
            float magnitude = (float)Math.Sqrt(delta.X * delta.X + delta.Y * delta.Y);
            if(magnitude > 0)
            {
                delta *= 5f / magnitude;
            }
            else
            {
                delta = new Vector2(0f, 5f);
            }
            Projectile.NewProjectile(npc.Center.X, npc.Center.Y, delta.X, delta.Y, ProjDef.byName["Bluemagic:ElementBall"].type, (npc.damage - 30) / 2, 3f, Main.myPlayer, GetDebuff(), GetDebuffTime());
            npc.netUpdate = true;
        }
    }

    public static void SetPosition(NPC npc)
    {
        Vector2 center = Main.npc[(int)npc.ai[0]].Center;
        double angle = Main.npc[(int)npc.ai[0]].ai[3] + 2.0 * Math.PI * npc.ai[1] / 5.0;
        npc.position = center + 300f * (new Vector2((float)Math.Cos(angle), (float)Math.Sin(angle))) - npc.Size / 2f;
    }

    public override void DealtPlayer(Player player, int hitDir, int dmgDealt, bool crit)
    {
        if(Main.rand.Next(2) == 0)
        {
            int debuff = GetDebuff();
            if(debuff >= 0)
            {
                player.AddBuff(GetDebuff(), GetDebuffTime(), true);
            }
        }
    }

    public int GetDebuff()
    {
        switch((int)npc.ai[1])
        {
        case 0:
            return 44;
        case 1:
            return BuffDef.byName["Bluemagic:EtherealFlames"];
        case 3:
            return 70;
        case 4:
            return 69;
        default:
            return -1;
        }
    }

    public int GetDebuffTime()
    {
        switch((int)npc.ai[1])
        {
        case 0:
            return 400;
        case 1:
            return 300;
        case 3:
            return 400;
        case 4:
            return 900;
        default:
            return -1;
        }
    }

    public Color? GetColor()
    {
        switch((int)npc.ai[1])
        {
        case 0:
            return new Color(0, 230, 230);
        case 1:
            return new Color(0, 153, 230);
        case 3:
            return new Color(0, 178, 0);
        case 4:
            return new Color(230, 192, 0);
        default:
            return null;
        }
    }
        

    public override void SelectFrame(int frameSize)
    {
        npc.frame.Y = (int)npc.ai[1] * frameSize;
        if(npc.ai[1] == 1f)
        {
            npc.alpha = 100;
        }
        if(npc.ai[2] < 50f)
        {
            npc.frame.Y += 5 * frameSize;
        }
    }
}}