using System;
using Microsoft.Xna.Framework;
using Terraria;
using TAPI;

namespace Bluemagic.NPCs {
public class BlockFish : Fish
{
    public BlockFish() : base() {}

    public override bool PreAI()
    {
        if(npc.ai[1] == 0f)
        {
            if(Main.netMode == 1)
            {
                return false;
            }
            if(Main.rand.Next(100) == 0)
            {
                npc.ai[1] = 3f;
            }
            else
            {
                npc.ai[1] = (float)(1 + Main.rand.Next(2));
            }
            npc.netUpdate = true;
        }
        if(npc.ai[1] == 1f)
        {
            npc.displayName = "Dirtfish";
            npc.damage = 110;
            npc.defense = 70;
        }
        else if(npc.ai[1] == 2f)
        {
            npc.displayName = "Stonefish";
            npc.damage = 100;
            npc.defense = 80;
        }
        else
        {
            npc.displayName = "True Goldfish";
            npc.damage = 200;
            npc.defense = 120;
            npc.value = 1000000;
        }
        return true;
    }

    public override bool CanSpawn(int x, int y, int type, Player spawnedOn)
    {
        int tile = (int)Main.tile[x, y].type;
        int wall = (int)Main.tile[x, y].wall;
        return (Bluemagic.NormalSpawn(y) && Bluemagic.NoBiome(spawnedOn, tile) && (!NPC.areaSky && !spawnedOn.zoneMeteor && !Bluemagic.IsBiomeBlock("Meteor", tile) && !NPC.areaNest && wall != 62 && NPC.areaWater)) && y > Main.worldSurface && y < Main.hellLayer && Bluemagic.downedAbomination > 0;
    }

    public override void HitEffect(int hitDirection, double damage, bool isDead)
    {
        if(npc.ai[0] == 1f)
        {
            Main.PlaySound(0, (int)npc.position.X, (int)npc.position.Y, 1);
        }
        else
        {
            Main.PlaySound(21, (int)npc.position.X, (int)npc.position.Y, 1);
        }
        int numDust = isDead ? 10 : 3;
        int dustType = 0;
        if(npc.ai[1] == 2f)
        {
            dustType = 1;
        }
        if(npc.ai[1] == 3f)
        {
            dustType = 10;
        }
        for(int k = 0; k < numDust; k++)
        {
            Dust.NewDust(npc.position, npc.width, npc.height, dustType, 0f, 0f, 0, default(Color), 1f);
        }
    }

    public override void SelectFrame(int frameSize)
    {
        base.SelectFrame(frameSize);
        npc.frame.Y += (int)(npc.ai[1] - 1f) * frameSize * 6;
    }

    public override void SetupLootRules(NPC unused)
    {
        bool flag = LootRule.settingUp;
        LootRule.settingUp = true;
        LootRule[] rules = new LootRule[5];
        rules[0] = new LootRule(null).Rules((NPC npc) => npc.ai[1] == 1f).LootRules(new LootRule[]
        {
            new LootRule(null).Item("Dirt Block")
        });
        rules[1] = new LootRule(null).Rules((NPC npc) => npc.ai[1] == 2f).LootRules(new LootRule[]
        {
            new LootRule(null).Item("Stone Block")
        });
        rules[2] = new LootRule(null).Rules((NPC npc) => npc.ai[1] == 3f).LootRules(new LootRule[]
        {
            new LootRule(null).Stack(30).Item("Copper Ore"),
            new LootRule(null).Stack(30).Item("Tin Ore"),
            new LootRule(null).Stack(30).Item("Iron Ore"),
            new LootRule(null).Stack(30).Item("Lead Ore"),
            new LootRule(null).Stack(30).Item("Silver Ore"),
            new LootRule(null).Stack(30).Item("Tungsten Ore"),
            new LootRule(null).Stack(30).Item("Gold Ore"),
            new LootRule(null).Stack(30).Item("Platinum Ore"),
            new LootRule(null).Stack(30).Item("Demonite Ore"),
            new LootRule(null).Stack(30).Item("Crimtane Ore"),
            new LootRule(null).Stack(30).Item("Cobalt Ore"),
            new LootRule(null).Stack(30).Item("Palladium Ore"),
            new LootRule(null).Stack(30).Item("Mythril Ore"),
            new LootRule(null).Stack(30).Item("Orichalcum Ore"),
            new LootRule(null).Stack(30).Item("Adamantite Ore"),
            new LootRule(null).Stack(30).Item("Titanium Ore"),
            new LootRule(null).Stack(60).Item("Chlorophyte Ore"),
            new LootRule(null).Stack(60).Item("Bluemagic:PuriumOre"),
            new LootRule(null).Item("Bluemagic:GoldenFishStatue")
        });
        rules[3] = new LootRule(null).Chance(0.1).Item("Goldfish");
        rules[4] = new LootRule(null).Chance(0.005).Item("Goldfish Banner");
        LootRule.AddFor("Bluemagic:BlockFish", rules);
        LootRule.settingUp = flag;
    }
}}