using System;
using Microsoft.Xna.Framework;
using Terraria;
using TAPI;

namespace Bluemagic.NPCs {
public class FallenZombie : Ghost
{
    public FallenZombie() : base(2.5f, 0.12f, 1.8f, 0.05f) {}

    public override bool CanSpawn(int x, int y, int type, Player spawnedOn)
    {
        int tile = (int)Main.tile[x, y].type;
        int wall = (int)Main.tile[x, y].wall;
        return Bluemagic.NoBiomeNormalSpawn(spawnedOn, tile, wall, y) && y > Main.worldSurface && y < Main.hellLayer && !NPC.areaTown && Bluemagic.downedAbomination > 0 && Main.rand.Next(4) == 0;
    }

    public override void DealtPlayer(Player player, int hitDir, int dmgDealt, bool crit)
    {
        if(Main.rand.Next(3) == 0)
        {
            player.AddBuff(23, 240, true);
        }
    }

    public override void HitEffect(int hitDirection, double damage, bool isDead)
    {
        if(isDead)
        {
            for(int k = 0; k < 50; k++)
            {
                int dust = Dust.NewDust(npc.position, npc.width, npc.height, 192, (float)(2 * hitDirection), -2f, 100, new Color(100, 100, 100, 100), 1f);
                Main.dust[dust].noGravity = true;
            }
        }
        else
        {
            int count = 0;
            while((double)count < damage / (double)npc.lifeMax * 100.0)
            {
                int dust = Dust.NewDust(npc.position, npc.width, npc.height, 192, (float)hitDirection, -1f, 100, new Color(100, 100, 100, 100), 1f);
                Main.dust[dust].noGravity = true;
                count++;
            }
        }
    }

    public override void SelectFrame(int frameSize)
    {
        npc.frameCounter += 1.0;
        npc.frameCounter %= 40.0;
        if(npc.frameCounter < 10.0)
        {
            npc.frame.Y = 0;
        }
        else if(npc.frameCounter < 20.0)
        {
            npc.frame.Y = frameSize;
        }
        else if(npc.frameCounter < 30.0)
        {
            npc.frame.Y = 0;
        }
        else
        {
            npc.frame.Y = 2 * frameSize;
        }
        npc.spriteDirection = npc.direction;
    }

    public override void SetupLootRules(NPC unused)
    {
        bool flag = LootRule.settingUp;
        LootRule.settingUp = true;
        LootRule[] rules = new LootRule[3];
        rules[0] = new LootRule(null).Chance(0.02).Item("Nazar");
        rules[1] = new LootRule(null).Chance(0.005).Item("Bluemagic:FallenZombieBanner");
        rules[2] = new LootRule(null).Chance(0.05).Item("Bluemagic:GhoulStaff");
        LootRule.AddFor("Bluemagic:FallenZombie", rules);
        LootRule.settingUp = flag;
    }
}}