using System;
using Microsoft.Xna.Framework;
using Terraria;
using TAPI;

namespace Bluemagic.NPCs {
public class GunnerSkeleton : Shooter
{
    public GunnerSkeleton() : base("Bluemagic:PuriumBulletH", 2f, 70, 45) {}

    public override bool CanSpawn(int x, int y, int type, Player spawnedOn)
    {
        int tile = (int)Main.tile[x, y].type;
        int wall = (int)Main.tile[x, y].wall;
        return Bluemagic.NoBiomeNormalSpawn(spawnedOn, tile, wall, y) && y > Main.worldSurface && y < Main.hellLayer && Bluemagic.downedAbomination > 0 && Main.rand.Next(2) == 0;
    }

    public override void HitEffect(int hitDirection, double damage, bool isDead)
    {
        if(isDead)
        {
            for(int k = 0; k < 10; k++)
            {
                int dust = Dust.NewDust(npc.position, npc.width, npc.height, 31, 0f, 0f, 100, default(Color), 1.5f);
                Main.dust[dust].velocity *= 1.4f;
            }
            for(int k = 0; k < 5; k++)
            {
                int dust = Dust.NewDust(npc.position, npc.width, npc.height, 6, 0f, 0f, 100, default(Color), 2.5f);
                Main.dust[dust].noGravity = true;
                Main.dust[dust].velocity *= 5f;
                dust = Dust.NewDust(npc.position, npc.width, npc.height, 6, 0f, 0f, 100, default(Color), 1.5f);
                Main.dust[dust].velocity *= 3f;
            }
            int gore = Gore.NewGore(new Vector2(npc.position.X, npc.position.Y - 10f), Vector2.Zero, Main.rand.Next(61, 64), npc.scale);
            Main.gore[gore].velocity *= 0.3f;
            gore = Gore.NewGore(new Vector2(npc.position.X, npc.position.Y + (float)(npc.height / 2) - 15f), Vector2.Zero, Main.rand.Next(61, 64), npc.scale);
            Main.gore[gore].velocity *= 0.3f;
            gore = Gore.NewGore(new Vector2(npc.position.X, npc.position.Y + (float)npc.height - 20f), Vector2.Zero, Main.rand.Next(61, 64), npc.scale);
            Main.gore[gore].velocity *= 0.3f;
        }
    }

    public override void SetupLootRules(NPC unused)
    {
        bool flag = LootRule.settingUp;
        LootRule.settingUp = true;
        LootRule[] rules = new LootRule[5];
        rules[0] = new LootRule(null).Rules(new Func<NPC, bool>[]{(NPC npc) => WorldGen.ironBar == 22}).Chance(0.5).Stack(1, 2).Item("Iron Bar");
        rules[1] = new LootRule(null).Rules(new Func<NPC, bool>[]{(NPC npc) => WorldGen.ironBar != 22}).Chance(0.5).Stack(1, 2).Item("Lead Bar");
        rules[2] = new LootRule(null).Chance(0.05).Item("Bluemagic:PuriumRifle");
        rules[3] = new LootRule(null).Chance(0.2).Stack(3, 5).Item("Bluemagic:PuriumBullet");
        rules[4] = new LootRule(null).Chance(0.005).Item("Bluemagic:MechaSkeletonBanner");
        LootRule.AddFor("Bluemagic:GunnerSkeleton", rules);
        LootRule.settingUp = flag;
    }
}}