using System;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Terraria;
using TAPI;

namespace Bluemagic.NPCs {
public class Turret : Caster
{
    public Turret() : base(300, new int[]{60, 90, 120, 150, 180}, 8f, "Bluemagic:RuneShot", 50, 20) {}

    public override bool PreAI()
    {
        if(npc.justHit)
        {
            npc.ai[0] = 290f;
        }
        return true;
    }

    public override void AI()
    {
        base.AI();
        Lighting.AddLight((int)(npc.Center.X / 16f), (int)(npc.position.Y / 16f), 0.7f, 0f, 0f);
    }

    public override bool CanSpawn(int x, int y, int type, Player spawnedOn)
    {
        int tile = (int)Main.tile[x, y].type;
        int wall = (int)Main.tile[x, y].wall;
        return Bluemagic.NoBiomeNormalSpawn(spawnedOn, tile, wall, y) && y > Main.worldSurface && y < Main.hellLayer && !NPC.areaTown && Bluemagic.downedAbomination > 0 && Main.rand.Next(10) == 0;
    }

    public override void HitEffect(int hitDirection, double damage, bool isDead)
    {
        if(isDead)
        {
            for(int k = 0; k < 10; k++)
            {
                int dust = Dust.NewDust(npc.position, npc.width, npc.height, 31, 0f, 0f, 100, default(Color), 1.5f);
                Main.dust[dust].velocity *= 1.4f;
            }
            for(int k = 0; k < 5; k++)
            {
                int dust = Dust.NewDust(npc.position, npc.width, npc.height, 6, 0f, 0f, 100, default(Color), 2.5f);
                Main.dust[dust].noGravity = true;
                Main.dust[dust].velocity *= 5f;
                dust = Dust.NewDust(npc.position, npc.width, npc.height, 6, 0f, 0f, 100, default(Color), 1.5f);
                Main.dust[dust].velocity *= 3f;
            }
            for(int k = 0; k < 15; k++)
            {
                BluemagicDust.CreateRuneDust(npc.position, npc.width, npc.height);
            }
            int gore = Gore.NewGore(new Vector2(npc.position.X, npc.position.Y - 10f), Vector2.Zero, Main.rand.Next(61, 64), npc.scale);
            Main.gore[gore].velocity *= 0.3f;
            gore = Gore.NewGore(new Vector2(npc.position.X, npc.position.Y + (float)(npc.height / 2) - 15f), Vector2.Zero, Main.rand.Next(61, 64), npc.scale);
            Main.gore[gore].velocity *= 0.3f;
            gore = Gore.NewGore(new Vector2(npc.position.X, npc.position.Y + (float)npc.height - 20f), Vector2.Zero, Main.rand.Next(61, 64), npc.scale);
            Main.gore[gore].velocity *= 0.3f;
        }
    }

    public override void SelectFrame(int frameSize)
    {
        if(npc.ai[0] >= 295f || npc.ai[0] <= 5f)
        {
            npc.frame.Y = 6 * frameSize;
        }
        else if(npc.ai[0] >= 290f || npc.ai[0] <= 10f)
        {
            npc.frame.Y = 5 * frameSize;
        }
        else
        {
            Player player = Main.player[npc.target];
            if(!player.active)
            {
                npc.frame.Y = 0;
            }
            else
            {
                Vector2 delta = npc.Center - player.Center;
                double angle = Math.Atan2(delta.Y, delta.X);
                if(angle > Math.PI * 0.375 && angle < Math.PI * 0.625)
                {
                    npc.frame.Y = 2 * frameSize;
                }
                else if(angle > Math.PI * 0.125 && angle < Math.PI * 0.875)
                {
                    npc.frame.Y = frameSize;
                }
                else if(angle > -Math.PI * 0.125 || angle < -Math.PI * 0.875)
                {
                    npc.frame.Y = 0;
                }
                else if(angle > -Math.PI * 0.375 || angle < -Math.PI * 0.625)
                {
                    npc.frame.Y = 3 * frameSize;
                }
                else
                {
                    npc.frame.Y = 4 * frameSize;
                }
            }
        }
        npc.spriteDirection = npc.direction;
    }

    public override void SetupLootRules(NPC unused)
    {
        bool flag = LootRule.settingUp;
        LootRule.settingUp = true;
        LootRule[] rules = new LootRule[5];
        rules[0] = new LootRule(null).Rules(new Func<NPC, bool>[]{(NPC npc) => WorldGen.ironBar == 22}).Chance(0.5).Stack(1, 2).Item("Iron Bar");
        rules[1] = new LootRule(null).Rules(new Func<NPC, bool>[]{(NPC npc) => WorldGen.ironBar != 22}).Chance(0.5).Stack(1, 2).Item("Lead Bar");
        rules[2] = new LootRule(null).Item("Bluemagic:WirelessTransceiver");
        rules[3] = new LootRule(null).Chance(0.005).Item("Bluemagic:SentinelBanner");
        rules[4] = new LootRule(null).Chance(0.1).Item("Bluemagic:RuneStaff");
        LootRule.AddFor("Bluemagic:Turret", rules);
        LootRule.settingUp = flag;
    }

    public override void PostDraw(SpriteBatch sb)
    {
        sb.Draw(modBase.textures["NPCs/Cave/TurretLight"], npc.position - Main.screenPosition + new Vector2(0f, 2f), new Rectangle(0, npc.frame.Y, npc.width, npc.height), Color.White, 0f, Vector2.Zero, 1f, SpriteEffects.None, 0f);
    }

    public override void TeleportDust()
    {
        for(int k = 0; k < 10; k++)
        {
            BluemagicDust.CreateRuneDust(npc.position, npc.width, npc.height);
        }
    }

    public override Vector2 ShootPos()
    {
        return npc.Center;
    }
}}