using System;
using Microsoft.Xna.Framework;
using Terraria;
using TAPI;

namespace Bluemagic.NPCs {
public class BabyFeeder : Worm
{
    public override bool CanSpawn(int x, int y, int type, Player spawnedOn)
    {
        if(type != NPCDef.byName["Bluemagic:BabyFeederHead"].type)
        {
            return false;
        }
        int tile = (int)Main.tile[x, y].type;
        int wall = (int)Main.tile[x, y].wall;
        return Bluemagic.NoZoneNormalSpawnAllowWater(spawnedOn, tile, wall, y) && Bluemagic.IsBiomeBlock("Corruption", tile) && y > Main.worldSurface && y < Main.hellLayer && !NPC.areaTown && Bluemagic.downedAbomination > 0 && Main.rand.Next(2) == 0;
    }

    public override void DealtPlayer(Player player, int hitDir, int dmgDealt, bool crit)
    {
        int healAmount = dmgDealt;
        if(!head)
        {
            if(tail)
            {
                healAmount = (int)(healAmount * 0.5f);
            }
            else
            {
                healAmount = (int)(healAmount * 0.75f);
            }
            if(healAmount == 0)
            {
                healAmount = 1;
            }
        }
        if(healAmount > npc.lifeMax - npc.life)
        {
            healAmount = npc.lifeMax - npc.life;
        }
        BabyFeeder worm = npc.GetSubClass<BabyFeeder>();
        while(worm != null && !worm.head)
        {
            worm = Main.npc[(int)worm.npc.ai[1]].GetSubClass<BabyFeeder>();
        }
        if(worm != null)
        {
            worm.npc.life += healAmount;
            BluemagicNet.SendNPCHeal(npc.whoAmI, healAmount);
        }
    }

    public override void HitEffect(int hitDirection, double damage, bool isDead)
    {
        if(isDead)
        {
            for(int k = 0; k < 10; k++)
            {
                Dust.NewDust(npc.position, npc.width, npc.height, 18, hitDirection, -1f, 0, default(Color), 1f);
            }
            if(head)
            {
                Gore.NewGore(npc.position, npc.velocity, "Bluemagic:BabyFeederHead", npc.scale);
            }
            else if(tail)
            {
                Gore.NewGore(npc.position, npc.velocity, "Bluemagic:BabyFeederTail", npc.scale);
            }
            else
            {
                Gore.NewGore(npc.position, npc.velocity, "Bluemagic:BabyFeederBody", npc.scale);
            }
        }
        else
        {
            for(int k = 0; k < 5; k++)
            {
                Dust.NewDust(npc.position, npc.width, npc.height, 18, hitDirection, -1f, 0, default(Color), 1f);
            }
        }
    }

    public override void SetupLootRules(NPC type)
    {
        bool flag = LootRule.settingUp;
        LootRule.settingUp = true;
        LootRule[] rules = new LootRule[2];
        rules[0] = new LootRule(null).Item("Cursed Flame");
        rules[1] = new LootRule(null).Chance(0.005).Item("Bluemagic:BabyFeederBanner");
        LootRule.AddFor(type.name, rules);
        LootRule.settingUp = flag;
    }
}}