using System;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Terraria;
using TAPI;

namespace Bluemagic.NPCs {
public class DarkTotem : ModNPC
{
    public override void AI()
    {
        npc.ai[0] += 1f;
        if(npc.ai[0] == 50f)
        {
            npc.netUpdate = true;
        }
        if(npc.ai[0] >= 100f)
        {
            if(Main.netMode != 1)
            {
                npc.TargetClosest(false);
                Player player = Main.player[npc.target];
                if(player.active && !player.dead && Vector2.Distance(npc.Center, player.Center) < 2400f)
                {
                    Vector2 shootPos = new Vector2(npc.Center.X + npc.width / 2, npc.position.Y + 16f);
                    Vector2 direction = player.Center - shootPos;
                    if(direction == Vector2.Zero)
                    {
                        direction = new Vector2(0f, -1f);
                    }
                    else
                    {
                        direction.Normalize();
                    }
                    Projectile.NewProjectile(shootPos, 8f * direction, "Bluemagic:DarkFlames", npc.damage / 2, 0f, Main.myPlayer);
                }
            }
            npc.ai[0] = 0f;
        }

        if(Main.rand.Next(2) == 0)
        {
            int dust = BluemagicDust.CreateDFlameDust(npc.position.X, npc.position.Y, 32, 32);
            Main.dust[dust].velocity.Y -= 2f;
        }
    }

    public override void SelectFrame(int frameSize)
    {
        npc.frameCounter += 1.0;
        if(npc.frameCounter >= 15.0)
        {
            npc.frameCounter = 0.0;
            npc.frame.Y = frameSize - npc.frame.Y;
        }
    }

    public override void HitEffect(int hitDirection, double damage, bool isDead)
    {
        Vector2 woodPos = new Vector2(npc.position.X, npc.position.Y + 32f);
        if(isDead)
        {
            for(int k = 0; k < 25; k++)
            {
                Dust.NewDust(woodPos, npc.width, npc.height, 191, hitDirection, -1f, 0, default(Color), 1.1f);
            }
            int gore = Gore.NewGore(new Vector2(npc.position.X, npc.position.Y - 10f), Vector2.Zero, Main.rand.Next(61, 64), npc.scale);
            Main.gore[gore].velocity *= 0.3f;
            gore = Gore.NewGore(new Vector2(npc.position.X, npc.position.Y + (float)(npc.height / 2) - 15f), Vector2.Zero, Main.rand.Next(61, 64), npc.scale);
            Main.gore[gore].velocity *= 0.3f;
            gore = Gore.NewGore(new Vector2(npc.position.X, npc.position.Y + (float)npc.height - 20f), Vector2.Zero, Main.rand.Next(61, 64), npc.scale);
            Main.gore[gore].velocity *= 0.3f;
        }
        else
        {
            for(int k = 0; k < damage / npc.lifeMax * 50.0; k++)
            {
                Dust.NewDust(woodPos, npc.width, npc.height, 191, hitDirection, -1f, 0, default(Color), 1.1f);
            }
        }
    }

    public override bool CanSpawn(int x, int y, int type, Player spawnedOn)
    {
        int tile = (int)Main.tile[x, y].type;
        int wall = (int)Main.tile[x, y].wall;
        return Bluemagic.NoZoneNormalSpawnAllowWater(spawnedOn, tile, wall, y) && Bluemagic.IsBiomeBlock("Corruption", tile) && y > Main.worldSurface && y < Main.hellLayer && !NPC.areaTown && Bluemagic.downedAbomination > 0 && Main.rand.Next(3) == 0;
    }

    public override void DealtPlayer(Player player, int hitDir, int dmgDealt, bool crit)
    {
        if(player.position.Y < npc.position.Y + 32f && Main.rand.Next(3) == 0)
        {
            player.AddBuff("Bluemagic:DarkFlames", 180, true);
        }
    }

    public override void SetupLootRules(NPC unused)
    {
        bool flag = LootRule.settingUp;
        LootRule.settingUp = true;
        LootRule[] rules = new LootRule[1];
        rules[0] = new LootRule(null).Chance(0.005).Item("Bluemagic:DarkTotemBanner");
        LootRule.AddFor("Bluemagic:DarkTotem", rules);
        LootRule.settingUp = flag;
    }
}}