using System;
using Microsoft.Xna.Framework;
using Terraria;
using TAPI;

namespace Bluemagic.NPCs {
public class FleshySpitter : Spitter
{
    public FleshySpitter() : base(d: 5, sh: "Bluemagic:FleshySpit") {}

    public override bool CanSpawn(int x, int y, int type, Player spawnedOn)
    {
        int tile = (int)Main.tile[x, y].type;
        int wall = (int)Main.tile[x, y].wall;
        return Bluemagic.NoZoneNormalSpawn(spawnedOn, tile, wall, y) && Bluemagic.IsBiomeBlock("Crimson", tile) && y < Main.hellLayer && Bluemagic.downedAbomination > 0 && Main.rand.Next(2) == 0;
    }

    public override void DealtPlayer(Player player, int hitDir, int dmgDealt, bool crit)
    {
        if(Main.rand.Next(8) == 0)
        {
            player.AddBuff("Bluemagic:Fleshy", 360, true);
            BluemagicNet.SendTransform(player.whoAmI, 3, 360);
        }
    }

    public override void HitEffect(int hitDirection, double damage, bool isDead)
    {
        if(isDead)
        {
            for(int k = 0; k < 50; k++)
            {
                Dust.NewDust(npc.position, npc.width, npc.height, 5, 2f * hitDirection, -2f, npc.alpha, default(Color), 1f);
            }
            Gore.NewGore(npc.position, npc.velocity, "Bluemagic:FleshySpitterFront", 1f);
            Gore.NewGore(npc.position, npc.velocity, "Bluemagic:FleshySpitterBack", 1f);
            Gore.NewGore(npc.position, npc.velocity, "Bluemagic:FleshySpitterSac", 1f);
        }
        else
        {
            for(int k = 0; k < damage / npc.lifeMax * 100.0; k++)
            {
                Dust.NewDust(npc.position, npc.width, npc.height, 5, hitDirection, -1f, npc.alpha, default(Color), 1f);
            }
        }
    }

    public override void SelectFrame(int frameSize)
    {
        npc.gfxOffY = 14f;
        npc.frameCounter += 1.0;
        npc.frameCounter %= 16.0;
        npc.frame.Y = frameSize * (int)(npc.frameCounter / 8.0);
    }

    public override void SetupLootRules(NPC unused)
    {
        bool flag = LootRule.settingUp;
        LootRule.settingUp = true;
        LootRule[] rules = new LootRule[2];
        rules[0] = new LootRule(null).Chance(0.33333333333333331).Item("Vertebrae");
        rules[1] = new LootRule(null).Chance(0.005).Item("Bluemagic:FleshySpitterBanner");
        LootRule.AddFor("Bluemagic:FleshySpitter", rules);
        LootRule.settingUp = false;
    }
}}