using System;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Terraria;
using TAPI;

namespace Bluemagic.NPCs {
public class ForeignEntity : Hover
{
    private string biome;
    private int buffType;
    private int buffTime;
    public ForeignEntity(string b, int bTy, int bTi) : base(1f, 0.1f, 1f, 0.025f)
    {
        biome = b;
        buffType = bTy;
        buffTime = bTi;
    }

    public override void OnSpawn()
    {
        npc.ai[3] += 180 * 30;
        npc.netUpdate = true;
    }

    public override void CustomBehavior(ref float ai)
    {
        AddLight();
        ai += 1f;
        if(ai % 180f == 0f)
        {
            if(Main.netMode != 1)
            {
                Vector2 direction = Main.player[npc.target].Center - npc.Center;
                if(direction.Length() < 750f)
                {
                    if(direction == Vector2.Zero)
                    {
                        direction = new Vector2(npc.direction, 0f);
                    }
                    else
                    {
                        direction.Normalize();
                    }
                    Projectile.NewProjectile(npc.Center, direction * 6f, "Bluemagic:ForeignBall", 50, 0f, Main.myPlayer, buffType, buffTime);
                }
                npc.netUpdate = true;
            }
            ai -= 180f;
        }
        if(((int)ai) / 180 < 600)
        {
            ai += 180f;
        }
        if(((int)ai) / 180 >= 600)
        {
            for(int k = 0; k < 255; k++)
            {
                Player player = Main.player[k];
                if(player.active && !player.dead && Math.Abs(player.Center.X - npc.Center.X) < 500 && Math.Abs(player.Center.Y - npc.Center.Y) < 400)
                {
                    Projectile.NewProjectile(player.Center, Vector2.Zero, "Bluemagic:PortalChaos", 60, 0f, Main.myPlayer);
                }
            }
            npc.netUpdate = true;
            ai -= 180 * 600;
            BluemagicNet.PlaySound(4, (int)npc.position.X, (int)npc.position.Y, 6);
        }
        if(!ShouldMove(ai))
        {
            if(Math.Abs(npc.velocity.X) >= 0.01f)
            {
                npc.velocity *= 0.95f;
            }
            else
            {
                npc.velocity.X = 0.01f * npc.direction;
            }
        }
        CreateDust();
    }

    public virtual void AddLight() {}

    public virtual void CreateDust() {}

    public override void SelectFrame(int frameSize)
    {
        npc.frameCounter += 1.0;
        if(npc.frameCounter >= 30.0)
        {
            npc.frameCounter = 0.0;
            npc.frame.Y = frameSize - npc.frame.Y;
        }
        npc.spriteDirection = -npc.direction;
    }

    public override bool ShouldMove(float ai)
    {
        return ai >= 180 * 30;
    }

    public override bool CanSpawn(int x, int y, int type, Player spawnedOn)
    {
        int tile = (int)Main.tile[x, y].type;
        int wall = (int)Main.tile[x, y].wall;
        return Bluemagic.NoZoneNormalSpawnAllowWater(spawnedOn, tile, wall, y) && Bluemagic.IsBiomeBlock(biome, tile) && y > Main.worldSurface && y < Main.hellLayer && !NPC.areaTown && Bluemagic.downedAbomination > 0 && Main.rand.Next(5) <= 1;
    }

    public override void DealtPlayer(Player player, int hitDir, int dmgDealt, bool crit)
    {
        if(Main.rand.Next(2) == 0)
        {
            player.AddBuff(buffType, buffTime, true);
        }
    }

    public override void PostDraw(SpriteBatch sb)
    {
        int counter = ((int)npc.ai[3]) / 180;
        if(counter < 30)
        {
            float strength = 0.9f - counter * 0.03f;
            Texture2D texture = Bluemagic.instance.textures["NPCs/Dark/ForeignEntityAttack"];
            SpriteEffects effects = npc.spriteDirection == 1 ? SpriteEffects.FlipHorizontally : SpriteEffects.None;
            sb.Draw(texture, npc.Center - Main.screenPosition + new Vector2(0f, 2f), npc.frame, Lighting.GetColor((int)(npc.Center.X / 16f), (int)(npc.Center.Y / 16f)) * strength, 0f, new Vector2(texture.Width / 2, texture.Height / 4), 1f, effects, 0f);
        }
    }
}}