using System;
using Microsoft.Xna.Framework;
using Terraria;
using TAPI;

namespace Bluemagic.NPCs {
public class RottingSpitter : Spitter
{
    public RottingSpitter() : base(d: 18, sh: "Bluemagic:RottingSpit") {}

    public override bool CanSpawn(int x, int y, int type, Player spawnedOn)
    {
        int tile = (int)Main.tile[x, y].type;
        int wall = (int)Main.tile[x, y].wall;
        return Bluemagic.NoZoneNormalSpawn(spawnedOn, tile, wall, y) && Bluemagic.IsBiomeBlock("Corruption", tile) && y < Main.hellLayer && Bluemagic.downedAbomination > 0 && Main.rand.Next(2) == 0;
    }

    public override void DealtPlayer(Player player, int hitDir, int dmgDealt, bool crit)
    {
        if(Main.rand.Next(8) == 0)
        {
            player.AddBuff("Bluemagic:Rotting", 360, true);
            BluemagicNet.SendTransform(player.whoAmI, 2, 360);
        }
    }

    public override void HitEffect(int hitDirection, double damage, bool isDead)
    {
        if(isDead)
        {
            for(int k = 0; k < 50; k++)
            {
                Dust.NewDust(npc.position, npc.width, npc.height, 18, hitDirection, -2f, npc.alpha, npc.color, npc.scale);
            }
            Gore.NewGore(npc.position, npc.velocity, 108, npc.scale);
            Gore.NewGore(npc.position, npc.velocity, 108, npc.scale);
            Gore.NewGore(npc.position, npc.velocity, "Bluemagic:RottingSpitterBody", npc.scale);
            Gore.NewGore(npc.position, npc.velocity, 110, npc.scale);
            Gore.NewGore(npc.position, npc.velocity, "Bluemagic:RottingSpitterSac", npc.scale);
        }
        else
        {
            for(int k = 0; k < damage / npc.lifeMax * 100.0; k++)
            {
                Dust.NewDust(npc.position, npc.width, npc.height, 18, hitDirection, -1f, npc.alpha, npc.color, npc.scale);
            }
        }
    }

    public override void SelectFrame(int frameSize)
    {
        npc.gfxOffY = 12f;
        npc.frameCounter += 1.0;
        npc.frameCounter %= 24.0;
        if(npc.frameCounter < 18.0)
        {
            npc.frame.Y = frameSize * (int)(npc.frameCounter / 6.0);
        }
        else
        {
            npc.frame.Y = frameSize;
        }
    }

    public override void SetupLootRules(NPC unused)
    {
        bool flag = LootRule.settingUp;
        LootRule.settingUp = true;
        LootRule[] rules = new LootRule[2];
        rules[0] = new LootRule(null).Chance(0.33333333333333331).Item("Rotten Chunk");
        rules[1] = new LootRule(null).Chance(0.005).Item("Bluemagic:RottingSpitterBanner");
        LootRule.AddFor("Bluemagic:RottingSpitter", rules);
        LootRule.settingUp = false;
    }
}}