using System;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Terraria;
using TAPI;

namespace Bluemagic.NPCs {
public class SuspiciousCoelom : ModNPC
{
    public override void AI()
    {
        npc.ai[0] += 1f;
        npc.ai[0] %= 300f;
        npc.ai[1] += 1f;
        npc.ai[1] %= 240f;
        float angle = 2f * (float)Math.PI * (1f - npc.ai[0] / 300f);
        float radius = 160f * (float)Math.Sin(2f * (float)Math.PI * npc.ai[1] / 240f);
        Vector2 offset = radius * new Vector2((float)Math.Cos(angle), (float)Math.Sin(angle));

        npc.TargetClosest(false);
        Player player = Main.player[npc.target];
        Vector2 targetVel = player.Center + offset - npc.Center;
        if(targetVel.Length() > 4f)
        {
            targetVel.Normalize();
            targetVel *= 4f;
        }
        npc.velocity = (4 * npc.velocity + targetVel) / 5f;
        npc.rotation = angle;
    }

    public override bool CanSpawn(int x, int y, int type, Player spawnedOn)
    {
        int tile = (int)Main.tile[x, y].type;
        int wall = (int)Main.tile[x, y].wall;
        return Bluemagic.NoZoneNormalSpawnAllowWater(spawnedOn, tile, wall, y) && Bluemagic.IsBiomeBlock("Crimson", tile) && y > Main.worldSurface && y < Main.hellLayer && !NPC.areaTown && Bluemagic.downedAbomination > 0 && Main.rand.Next(3) == 0;
    }

    public override void HitEffect(int hitDirection, double damage, bool isDead)
    {
        Color color = new Color(220, 220, 90);
        if(isDead)
        {
            for(int k = 0; k < 25; k++)
            {
                Dust.NewDust(npc.position, npc.width, npc.height, 4, 2f * hitDirection, -2f, npc.alpha, color, 1f);
            }
        }
        else
        {
            for(int k = 0; k < damage / npc.lifeMax * 50; k++)
            {
                Dust.NewDust(npc.position, npc.width, npc.height, 4, hitDirection, -1f, npc.alpha, color, 1f);
            }
        }
    }

    public override void DealtPlayer(Player player, int hitDir, int dmgDealt, bool crit)
    {
        if(Main.rand.Next(2) == 0)
        {
            player.AddBuff("Bluemagic:Sick", 600, true);
        }
    }

    public override void SetupLootRules(NPC unused)
    {
        bool flag = LootRule.settingUp;
        LootRule.settingUp = true;
        LootRule[] rules = new LootRule[1];
        rules[0] = new LootRule(null).Chance(0.005).Item("Bluemagic:SuspiciousCoelomBanner");
        LootRule.AddFor("Bluemagic:SuspiciousCoelom", rules);
        LootRule.settingUp = flag;
    }

    public override bool PreDraw(SpriteBatch sb)
    {
        sb.Draw(Bluemagic.instance.textures["NPCs/Dark/SuspiciousCoelomBack"], npc.Center - Main.screenPosition + new Vector2(0f, 4f), null, Lighting.GetColor((int)(npc.Center.X / 16f), (int)(npc.Center.Y / 16f)), npc.rotation, new Vector2(npc.width / 2, npc.height / 2), 1f, SpriteEffects.None, 0f);
        return true;
    }

    public override void PostDraw(SpriteBatch sb)
    {
        sb.Draw(Bluemagic.instance.textures["NPCs/Dark/SuspiciousCoelomFront"], npc.Center - Main.screenPosition + new Vector2(0f, 4f), null, Lighting.GetColor((int)(npc.Center.X / 16f), (int)(npc.Center.Y / 16f)), npc.rotation, new Vector2(npc.width / 2, npc.height / 2), 1f, SpriteEffects.None, 0f);
    }
}}