using System;
using Microsoft.Xna.Framework;
using Terraria;
using TAPI;

namespace Bluemagic.NPCs {
public class InfectedMummy : Fighter
{
    public InfectedMummy() : base(1f, 0.05f, true, 26, 1, 500, true, false) {}

    public override void AI()
    {
        if(npc.life < npc.lifeMax / 2)
        {
            speed = 2f;
            acceleration = 0.1f;
        }
        base.AI();
    }

    public override bool CanSpawn(int x, int y, int type, Player spawnedOn)
    {
        Tile tile = Main.tile[x, y];
        return Bluemagic.NoZoneNormalSpawn(spawnedOn, tile.type, tile.wall, y) && Bluemagic.IsBiomeBlock("Desert", tile.type) && Bluemagic.IsBiomeBlock("Glowshroom", tile.type) && Main.hardMode && Main.rand.Next(3) != 0;
    }

    public override void DealtPlayer(Player player, int hitDir, int dmgDealt, bool crit)
    {
        if(Main.rand.Next(8) == 0)
        {
            player.AddBuff(32, 900, true);
        }
        if(Main.rand.Next(14) == 0)
        {
            player.AddBuff(31, 300, true);
        }
    }

    public override void HitEffect(int hitDirection, double damage, bool isDead)
    {
        if(isDead)
        {
            for(int k = 0; k < 20; k++)
            {
                int dust = Dust.NewDust(npc.position, npc.width, npc.height, 31, 0f, 0f, 0, default(Color), 1.5f);
                Main.dust[dust].velocity *= 2f;
                Main.dust[dust].noGravity = true;
            }
            int gore = Gore.NewGore(new Vector2(npc.position.X, npc.position.Y - 10f), new Vector2((float)hitDirection, 0f), 61, npc.scale);
            Main.gore[gore].velocity *= 0.3f;
            gore = Gore.NewGore(new Vector2(npc.position.X, npc.position.Y + (float)(npc.height / 2) - 10f), new Vector2((float)hitDirection, 0f), 62, npc.scale);
            Main.gore[gore].velocity *= 0.3f;
            gore = Gore.NewGore(new Vector2(npc.position.X, npc.position.Y + (float)npc.height - 10f), new Vector2((float)hitDirection, 0f), 63, npc.scale);
            Main.gore[gore].velocity *= 0.3f;
        }
        else
        {
            for(int k = 0; k < damage / (double)npc.lifeMax * 50.0; k++)
            {
                int dust = Dust.NewDust(npc.position, npc.width, npc.height, 31, 0f, 0f, 0, default(Color), 1.5f);
                Main.dust[dust].velocity *= 2f;
                Main.dust[dust].noGravity = true;
            }
        }
    }

    public override void SetupLootRules(NPC unused)
    {
        bool flag = LootRule.settingUp;
        LootRule.settingUp = true;
        LootRule[] rules = new LootRule[7];
        rules[0] = new LootRule(null).Chance(0.1).Item("Mushroom Grass Seeds");
        rules[1] = new LootRule(null).Chance(0.01).Item("Fast Clock");
        rules[2] = new LootRule(null).Chance(0.01).Item("Trifold Map");
        rules[3] = new LootRule(null).Chance(0.013333333333333334).Item("Mummy Mask");
        rules[4] = new LootRule(null).Chance(0.013333333333333334).Item("Mummy Shirt");
        rules[5] = new LootRule(null).Chance(0.013333333333333334).Item("Mummy Pants");
        rules[6] = new LootRule(null).Chance(0.005).Item("Mummy Banner");
        LootRule.AddFor("Bluemagic:InfectedMummy", rules);
        LootRule.settingUp = flag;
    }
}}