using System;
using Microsoft.Xna.Framework;
using Terraria;
using TAPI;

namespace Bluemagic.NPCs {
public class MotherScorpion : Shooter
{
    public MotherScorpion() : base(55, 8f, 100, 0, 2, 17) {}

    public override Vector2 ShootOffset()
    {
        return new Vector2(-2f * npc.direction, -6f);
    }

    public override bool CanSpawn(int x, int y, int type, Player spawnedOn)
    {
        Tile tile = Main.tile[x, y];
        return Bluemagic.NoZoneNormalSpawn(spawnedOn, tile.type, tile.wall, y) && Bluemagic.IsBiomeBlock("Desert", tile.type) && Bluemagic.downedAbomination > 0 && Main.rand.Next(3) != 0;
    }

    public override void SelectFrame(int frameSize)
    {
        if(npc.velocity.X == 0f)
        {
            npc.frame.Y = 0;
            npc.frameCounter = 0.0;
        }
        else
        {
            npc.frameCounter += 1.0 + Math.Abs(npc.velocity.X);
            if(npc.frameCounter > 6.0)
            {
                npc.frame.Y += frameSize;
                npc.frame.Y %= frameSize * Main.npcFrameCount[npc.type];
                npc.frameCounter = 0.0;
            }
            if(npc.direction == 1)
            {
                npc.spriteDirection = 1;
            }
            else if(npc.direction == -1)
            {
                npc.spriteDirection = -1;
            }
        }
    }

    public override void HitEffect(int hitDirection, double damage, bool isDead)
    {
        if(isDead)
        {
            for(int k = 0; k < 20; k++)
            {
                Dust.NewDust(npc.position, npc.width, npc.height, 203, 2f * hitDirection, -2f, 0, default(Color), 1f);
            }
            Gore.NewGore(npc.position, npc.velocity, "Bluemagic:MotherScorpionClaw", 1f);
            Gore.NewGore(npc.position, npc.velocity, "Bluemagic:MotherScorpion", 1f);
            Gore.NewGore(npc.position, npc.velocity, "Bluemagic:MotherScorpionTail", 1f);
        }
        else
        {
            for(int k = 0; k < damage / (double)npc.lifeMax * 40.0; k++)
            {
                Dust.NewDust(npc.position, npc.width, npc.height, 203, (float)hitDirection, -1f, 0, default(Color), 1f);
            }
        }
    }

    public override void SetupLootRules(NPC unused)
    {
        bool flag = LootRule.settingUp;
        LootRule.settingUp = true;
        LootRule[] rules = new LootRule[3];
        rules[0] = new LootRule(null).Chance(0.5).Item("Stinger");
        rules[1] = new LootRule(null).Chance(0.1).Item("Scorpion");
        rules[2] = new LootRule(null).Chance(0.005).Item("Bluemagic:MotherScorpionBanner");
        LootRule.AddFor("Bluemagic:MotherScorpion", rules);
        LootRule.settingUp = flag;
    }
}}