using System;
using Microsoft.Xna.Framework;
using Terraria;
using TAPI;

namespace Bluemagic.NPCs {
public class SandWorm : Worm
{
    public override void CustomBehavior()
    {
        if(head && Main.netMode != 1)
        {
            Player player = Main.player[npc.target];
            if(npc.localAI[2] < 200f)
            {
                npc.localAI[2] += 1f;
            }
            else if(Collision.CanHit(npc.position, npc.width, npc.height, player.position, player.width, player.height))
            {
                Vector2 offset = player.Center - npc.Center;
                float angle = (float)Math.Atan2(offset.Y, offset.X);
                if(angle < 0f)
                {
                    angle += 2f * (float)Math.PI;
                }
                float facing = npc.rotation - (float)Math.PI / 2f;
                if(facing < 0f)
                {
                    facing += 2f * (float)Math.PI;
                }
                float angleOffset = Math.Abs(angle - facing);
                if(angleOffset <= (float)Math.PI / 16f || angleOffset >= (float)Math.PI * 31f / 16f)
                {
                    if(offset != Vector2.Zero)
                    {
                        offset.Normalize();
                    }
                    int proj = Projectile.NewProjectile(npc.Center, offset * 12f, 31, npc.damage / 2, 0f, Main.myPlayer, 0f, 0f);
                    Main.projectile[proj].ai[0] = 2f;
                    Main.projectile[proj].timeLeft = 300;
                    Main.projectile[proj].friendly = false;
                    NetMessage.SendData(27, -1, -1, "", proj, 0f, 0f, 0f, 0f);
                    npc.localAI[2] = 0f;
                    BluemagicNet.PlaySound(2, (int)npc.position.X, (int)npc.position.Y, 5);
                }
            }
        }
    }

    public override bool CanSpawn(int x, int y, int type, Player spawnedOn)
    {
        if(type != NPCDef.byName["Bluemagic:SandWormHead"].type)
        {
            return false;
        }
        Tile tile = Main.tile[x, y];
        return Bluemagic.NoZoneNormalSpawn(spawnedOn, tile.type, tile.wall, y) && Bluemagic.IsBiomeBlock("Desert", tile.type) && !NPC.areaTown && Bluemagic.downedAbomination > 0 && Main.rand.Next(3) != 0;
    }

    public override void HitEffect(int hitDirection, double damage, bool isDead)
    {
        if(isDead)
        {
            for(int k = 0; k < 10; k++)
            {
                Dust.NewDust(npc.position, npc.width, npc.height, 5, 2.5f * (float)hitDirection, -2.5f, 0, default(Color), 1f);
            }
            if(head)
            {
                Gore.NewGore(npc.position, npc.velocity, 97, 1f);
                Gore.NewGore(npc.position, npc.velocity, "Bluemagic:SandWormHead", 1f);
            }
            else
            {
                Gore.NewGore(npc.position, npc.velocity, "Bluemagic:SandWormBody", 1f);
                Gore.NewGore(npc.position + new Vector2(6f, 0f), npc.velocity, "Bluemagic:SandWormBody", 1f);
            }
        }
        else
        {
            for(int k = 0; k < damage / (double)npc.lifeMax * 50.0; k++)
            {
                Dust.NewDust(npc.position, npc.width, npc.height, 5, (float)hitDirection, -1f, 0, default(Color), 1f);
            }
        }
    }

    public override void SelectFrame(int frameSize)
    {
        npc.gfxOffY = 15f;
    }

    public override void SetupLootRules(NPC type)
    {
        bool flag = LootRule.settingUp;
        LootRule.settingUp = true;
        LootRule[] rules = new LootRule[3];
        rules[0] = new LootRule(null).Chance(0.25).Item("Antlion Mandible");
        rules[1] = new LootRule(null).Item("Sand Block");
        rules[2] = new LootRule(null).Chance(0.005).Item("Bluemagic:SandWormBanner");
        LootRule.AddFor(type.name, rules);
        LootRule.settingUp = flag;
    }
}}