using System;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Terraria;
using TAPI;

namespace Bluemagic.NPCs {
public class Sarcophagus : Hover
{
    public Sarcophagus() : base(2f, 0.1f, 1f, 0.1f) {}

    public override void CustomBehavior(ref float ai)
    {
        Player player = Main.player[npc.target];
        ai += 1f;
        if(Math.Abs(npc.Center.X - player.Center.X) < 16f * 30f && Math.Abs(npc.Center.Y - player.Center.Y) < 16f * 20f)
        {
            if(!player.buffImmune[23] && ai >= 120f)
            {
                ai = -60f;
                if(Main.netMode != 1)
                {
                    BluemagicNet.PlaySound(4, (int)npc.position.X, (int)npc.position.Y, 6);
                }
                npc.netUpdate = true;
            }
            else if(ai >= 180f)
            {
                ai = -120f;
                if(Main.netMode != 1)
                {
                    BluemagicNet.PlaySound(4, (int)npc.position.X, (int)npc.position.Y, 6);
                    int proj = Projectile.NewProjectile(npc.Center, Vector2.Zero, "Bluemagic:ShadowArm", npc.damage / 2, 0f, Main.myPlayer, 60f, 0f);
                    Projectiles.ShadowArm arm = Main.projectile[proj].GetSubClass<Projectiles.ShadowArm>();
                    arm.target = player.Center;
                    arm.Add(npc.Center);
                    Vector2 offset = player.Center - npc.Center;
                    float angle = (float)Math.Atan2(offset.Y, offset.X);
                    if(offset.X <= 0f)
                    {
                        angle += (float)Math.PI * 0.875f;
                    }
                    else
                    {
                        angle -= (float)Math.PI * 0.875f;
                    }
                    Vector2 targetOffset = 16f * new Vector2((float)Math.Cos(angle), (float)Math.Sin(angle));
                    arm.Add(npc.Center + targetOffset);
                    if(Main.netMode == 2)
                    {
                        BluemagicNet.SendShadowArmData(proj);
                        BluemagicNet.SendTrailAdd(proj, npc.Center);
                        BluemagicNet.SendTrailAdd(proj, npc.Center + targetOffset);
                    }
                }
                npc.netUpdate = true;
            }
        }
        else if(ai > 300f)
        {
            ai = 300f;
        }
        if(ai < 0f)
        {
            if(Math.Abs(npc.velocity.X) >= 0.01f)
            {
                npc.velocity *= 0.95f;
            }
            else
            {
                npc.velocity.X = 0.01f * npc.direction;
            }
            if(ai == -1f)
            {
                for(int k = 0; k < 255; k++)
                {
                    Player target = Main.player[k];
                    if(Math.Abs(npc.Center.X - target.Center.X) < 16f * 30f && Math.Abs(npc.Center.Y - target.Center.Y) < 16f * 20f)
                    {
                        bool flag = target.AddBuff(23, 240, true) >= 0;
                        flag = target.AddBuff(32, 240, true) >= 0 || flag;
                        flag = target.AddBuff(22, 240, true) >= 0 || flag;
                        if(flag)
                        {
                            target.GetSubClass<BluemagicPlayer>().lockTime = 60;
                        }
                    }
                }
            }
            if(ai == -61f)
            {
                ai = -1f;
            }
        }

        for(int k = 0; k < 2; k++)
        {
            int dust = BluemagicDust.CreateSmoke(npc.position - new Vector2(8f, 8f), npc.width + 16, npc.height + 16, Color.Black);
            Main.dust[dust].velocity += npc.velocity * 0.25f;
        }
    }

    public override void SelectFrame(int frameSize)
    {
        if(npc.ai[3] % 30f == 0f)
        {
            npc.rotation = Main.rand.Next(-2, 3) * (float)Math.PI / 32f;
        }
        npc.spriteDirection = npc.direction;
    }

    public override bool ShouldMove(float ai)
    {
        return ai >= 0;
    }

    public override bool CanSpawn(int x, int y, int type, Player spawnedOn)
    {
        Tile tile = Main.tile[x, y];
        return Bluemagic.NoZoneNormalSpawn(spawnedOn, tile.type, tile.wall, y) && Bluemagic.IsBiomeBlock("Desert", tile.type) && !NPC.areaTown && Bluemagic.downedAbomination > 0 && Main.rand.Next(3) != 0;
    }

    public override void DealtPlayer(Player player, int hitDir, int dmgDealt, bool crit)
    {
        if(Main.rand.Next(3) == 0)
        {
            player.AddBuff(23, 240, true);
        }
    }

    public override void PostDraw(SpriteBatch sb)
    {
        if(npc.ai[3] < 0f && npc.ai[3] >= -60f)
        {
            float angle = npc.ai[3] / 30f * (float)Math.PI;
            sb.Draw(Bluemagic.instance.textures["NPCs/Desert/Seal"], npc.Center - Main.screenPosition + new Vector2(0f, 10f), null, Lighting.GetColor((int)(npc.Center.X / 16f), (int)(npc.Center.Y / 16f)) * 0.9f, angle, new Vector2(16f, 16f), 1f, SpriteEffects.None, 0f);
        }
    }

    public override void SetupLootRules(NPC unused)
    {
        bool flag = LootRule.settingUp;
        LootRule.settingUp = true;
        LootRule[] rules = new LootRule[2];
        rules[0] = new LootRule(null).Chance(0.02).Item("Nazar");
        rules[1] = new LootRule(null).Chance(0.005).Item("Bluemagic:SarcophagusBanner");
        LootRule.AddFor("Bluemagic:Sarcophagus", rules);
        LootRule.settingUp = flag;
    }
}}