using System;
using System.Collections.Generic;
using Terraria;
using TAPI;

namespace Bluemagic.NPCs {
[GlobalModAttribute]
public class GlobalNPC : ModNPC
{
    public override void SetupLootRules(NPC unused)
    {
        bool flag = LootRule.settingUp;
        LootRule.settingUp = true;

        LootRule[] rules = new LootRule[1];
        rules[0] = new LootRule("Rune Drop");
        Func<NPC, bool>[] conditions = new Func<NPC, bool>[5];
        conditions[0] = ((NPC npc) => npc.name != "Vanilla:Slimer");
        conditions[1] = LootRule.ruleHardmode;
        conditions[2] = LootRule.ruleHostile;
        conditions[3] = LootRule.ruleHasValue;
        conditions[4] = ((NPC npc) => NPC.downedMechBossAny);
        rules[0].Rules(conditions);
        conditions = new Func<NPC, bool>[3];
        conditions[0] = ((NPC npc) => (double)npc.position.Y <= Main.rockLayer * 16.0);
        conditions[1] = ((NPC npc) => !Main.bloodMoon && !Main.eclipse);
        conditions[2] = ((NPC npc) => Bluemagic.NoInvasion((int)(npc.position.Y / 16)));
        rules[0].LootRules(new LootRule[]
        {
            new LootRule("Solar/Lunar Drop").Rules(conditions).LootRules(new LootRule[]
            {
                new LootRule().Chance(0.25).Rules(new Func<NPC, bool>[]{(NPC npc) => Main.dayTime}).Item("Bluemagic:SolarDrop"),
                new LootRule().Chance(0.2).Rules(new Func<NPC, bool>[]{(NPC npc) => !Main.dayTime}).Item("Bluemagic:LunarDrop")
            }),
            new LootRule().Chance(0.1).Rules(new Func<NPC, bool>[]{(NPC npc) => Main.bloodMoon || (Main.eclipse && npc.position.Y <= Main.rockLayer * 16.0)}).Item("Bluemagic:HorrorDrop")
        });
        LootRule.Add(rules);

        rules = new LootRule[1];
        LootRule frostMoon = new LootRule("Frost Moon").Rules(new Func<NPC, bool>[]{LootRule.ruleFrostMoon});
        Func<NPC, bool>[] small = {((NPC npc) => NPC.waveCount >= 11 && NPC.waveCount < 15)};
        Func<NPC, bool>[] large = {((NPC npc) => NPC.waveCount >= 15)};
        LootRule[] conditionals = new LootRule[2];
        conditionals[0] = new LootRule(null).Rules(small).Chance((NPC npc) => 0.2 * (NPC.waveCount - 10)).Item("Bluemagic:Icicle");
        conditionals[1] = new LootRule(null).Rules(large).Stack(4, 6).Item("Bluemagic:Icicle");
        rules[0] = frostMoon.LootRules(conditionals);
        LootRule.AddFor("Ice Queen", rules);

        rules = new LootRule[1];
        rules[0] = new LootRule(null).Item("Bluemagic:Bubble").Stack(5, 7);
        LootRule.AddFor("Duke Fishron", rules);

        rules = new LootRule[1];
        LootRule pumpkinMoon = new LootRule("Pumpkin Moon").Rules(new Func<NPC, bool>[]{LootRule.rulePumpkinMoon});
        small = new Func<NPC, bool>[]{((NPC npc) => NPC.waveCount >= 11 && NPC.waveCount < 15)};
        large = new Func<NPC, bool>[]{((NPC npc) => NPC.waveCount >= 15)};
        conditionals = new LootRule[2];
        conditionals[0] = new LootRule(null).Rules(small).Chance((NPC npc) => 0.2 * (NPC.waveCount - 10)).Item("Bluemagic:ScytheBlade");
        conditionals[1] = new LootRule(null).Rules(large).Stack(4, 6).Item("Bluemagic:ScytheBlade");
        rules[0] = pumpkinMoon.LootRules(conditionals);
        LootRule.AddFor("Pumpking", rules);

        LootRule.settingUp = flag;
    }

    public override void UpdateSpawnRate(Player player)
    {
        if(((!Main.pumpkinMoon && !Main.snowMoon) || (double)(player.position.Y / 16f) >= Main.worldSurface) && !NPC.areaInvasion && Bluemagic.downedAbomination > 0)
        {
            NPC.spawnRate = (int)(NPC.spawnRate * 0.8);
            NPC.maxSpawns = (int)(NPC.maxSpawns * 1.2);
            int x = (int)(player.position.X / 16f);
            int y = (int)(player.position.Y / 16f);
            if(y < Main.worldSurface * 16 && (player.zone["Desert"] || x < 250 || x > Main.maxTilesX - 250 || player.zoneHoly))
            {
                NPC.spawnRate = (int)(NPC.spawnRate * 2f / 3f);
                NPC.maxSpawns++;
            }
        }
    }

    public override List<int> EditSpawnPool(int x, int y, List<int> pool, Player player)
    {
        Tile tile = Main.tile[x, y];
        if(pool.Contains(376))
        {
            pool.Clear();
            pool.Add(376);
            return pool;
        }
        if(!pool.Contains(68) && Bluemagic.NormalSpawn(y) && Bluemagic.NoZone(player, tile.type, tile.wall))
        {
            if(Bluemagic.IsBiomeBlock("Glowshroom", tile.type) && tile.type != 70 && tile.type != 71 && tile.type != 72)
            {
                if(pool.Count > 0 && pool[0] <= 377)
                {
                    pool.RemoveAt(0);
                }
                if(Main.hardMode && NPC.areaWater)
                {
                    pool.Add(256);
                }
                else if(y <= Main.worldSurface && Main.rand.Next(3) != 0)
                {
                    if((!Main.hardMode && Main.rand.Next(6) == 0) || Main.rand.Next(12) == 0)
                    {
                        pool.Add(360);
                    }
                    else if(Main.rand.Next(3) == 0)
                    {
                        if(Main.rand.Next(4) == 0)
                        {
                            if(Main.hardMode && Main.rand.Next(3) != 0)
                            {
                                pool.Add(-2);
                            }
                            else
                            {
                                pool.Add(-3);
                            }
                        }
                        else
                        {
                            pool.Add(257 + Main.rand.Next(2));
                        }
                    }
                    else
                    {
                        pool.Add(254 + Main.rand.Next(2));
                    }
                }
                else if(y >= Main.worldSurface && Main.rand.Next(3) != 0)
                {
                    if(Main.hardMode && Main.rand.Next(7) == 0)
                    {
                        pool.Add(374);
                    }
                    else if((!Main.hardMode && Main.rand.Next(4) == 0) || Main.rand.Next(8) == 0)
                    {
                        pool.Add(360);
                    }
                    else if(Main.rand.Next(4) == 0)
                    {
                        if(Main.hardMode && Main.rand.Next(3) != 0)
                        {
                            pool.Add(-2);
                        }
                        else
                        {
                            pool.Add(-3);
                        }
                    }
                    else
                    {
                        pool.Add(257 + Main.rand.Next(2));
                    }
                }
            }
        }
        for(int k = 0; k < pool.Count; k++)
        {
            if(pool[k] == NPCDef.byName["Bluemagic:MotherScorpion"].type && Main.rand.Next(2) == 0)
            {
                pool[k] = NPCDef.byName["Bluemagic:BlackMotherScorpion"].type;
            }
        }
        return pool;
    }
}}