using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Terraria;
using TAPI;

namespace Bluemagic.NPCs {
public class HardmodeGuide : ModNPC
{
    public override bool CanTownNPCSpawn()
    {
        return Main.hardMode;
    }

    public override void SelectFrame(int frameSize)
    {
        if (npc.velocity.Y == 0f)
        {
            if (npc.direction == 1)
            {
                npc.spriteDirection = 1;
            }
            if (npc.direction == -1)
            {
                npc.spriteDirection = -1;
            }
            if (npc.velocity.X == 0f)
            {
                npc.frame.Y = 0;
                npc.frameCounter = 0.0;
            }
            else
            {
                npc.frameCounter += (double)(System.Math.Abs(npc.velocity.X) * 2f);
                npc.frameCounter += 1.0;
                if (npc.frameCounter > 6.0)
                {
                    npc.frame.Y += frameSize;
                    npc.frameCounter = 0.0;
                }
                if (npc.frame.Y / frameSize >= Main.npcFrameCount[npc.type])
                {
                    npc.frame.Y = frameSize;
                }
            }
        }
        else
        {
            npc.frameCounter = 0.0;
            npc.frame.Y = frameSize;
        }
    }

    public override void HitEffect(int hitDirection, double damage, bool isDead)
    {
        if(isDead)
        {
            for (int k = 0; k < 50; k++)
            {
                Dust.NewDust(npc.position, npc.width, npc.height, 5, 2.5f * (float)hitDirection, -2.5f, 0, default(Color), 1f);
            }
            Gore.NewGore(npc.position, npc.velocity, "Bluemagic:HardmodeGuide1", 1f);
            Gore.NewGore(new Vector2(npc.position.X, npc.position.Y + 20f), npc.velocity, "Bluemagic:HardmodeGuide2", 1f);
            Gore.NewGore(new Vector2(npc.position.X, npc.position.Y + 20f), npc.velocity, "Bluemagic:HardmodeGuide2", 1f);
            Gore.NewGore(new Vector2(npc.position.X, npc.position.Y + 34f), npc.velocity, "Bluemagic:HardmodeGuide3", 1f);
            Gore.NewGore(new Vector2(npc.position.X, npc.position.Y + 34f), npc.velocity, "Bluemagic:HardmodeGuide3", 1f);
        }
        else
        {
            int count = 0;
            while((double)count < damage/(double)npc.lifeMax * 100.0)
            {
                Dust.NewDust(npc.position, npc.width, npc.height, 5, (float)hitDirection, -1f, 0, default(Color), 1f);
                count++;
            }
        }
    }

    public override string SetName()
    {
        switch(Main.rand.Next(7))
        {
            case 0:
                return "Alice";
            case 1:
                return "Brianna";
            case 2:
                return "Faith";
            case 3:
                return "Jessica";
            case 4:
                return "Natalia";
            case 5:
                return "Samantha";
            default:
                return "Zinnia";
        }
    }

    public override string SetChat()
    {
        if(Main.eclipse)
        {
            return "When the sun goes dark... then a bunch of gross monsters swarm everywhere.";
        }
        if(Main.bloodMoon)
        {
            return "...Really? You want advice at a time like npc?! Just hurry up...";
        }
        List<string> list = new List<string>();
        list.Add("The land of Terraria has a very deep history... I'll try to use my knowledge to offer advice.");
        list.Add("I suppose I could try to help save Terraria... but I'm too busy guiding, isn't that right?");
        if(NPC.AnyNPCs(22))
        {
            list.Add("They say there is a person who will tell you how to survive in this land... well, it certainly isn't " + NPC.firstNPCName(22) + "!");
        }
        if(NPC.AnyNPCs(229))
        {
            list.Add("I don't understand why the pirates keep attacking when " + NPC.firstNPCName(229) + " already joined us.");
        }
        if(NPC.AnyNPCs(160))
        {
            list.Add("People should really stop picking on " + NPC.firstNPCName(160) + ". It's not like he tastes good or anything.... Oh, not that I'd know myself, of course.");
        }
        return list[Main.rand.Next(list.Count)];
    }

    public override void SetChatButtons(ref string[] buttons)
    {
        buttons[0] = "Help";
        buttons[1] = "Crafting";
    }

    public override void OnChatButtonClicked(string[] buttons, int buttonIndex, ref bool openShop)
    {
        if(buttonIndex == 0)
        {
            Main.PlaySound(12, -1, -1, 1);
            Main.npcChatText = NextHelpText();
        }
        else if(buttonIndex == 1)
        {
            Main.playerInventory = true;
            Main.npcChatText = "";
            Main.PlaySound(12, -1, -1, 1);
            Main.craftGuide = true;
        }
    }

    private string NextHelpText()
    {
        string text = GetHelpText();
        Bluemagic.helpText++;
        Bluemagic.helpText %= 34;
        return text;
    }

    private string GetHelpText()
    {
        string dark = "";
        if(WorldGen.corruption)
        {
            dark = "corruption";
        }
        if(WorldGen.crimson)
        {
            dark = "crimson";
        }
        while(true)
        {
            switch(Bluemagic.helpText)
            {
            case 0:
                if(WorldGen.altarCount > 0)
                {
                    goto case -1;
                }
                string altar = "";
                if(WorldGen.corruption)
                {
                    altar = "demon";
                }
                if(WorldGen.crimson)
                {
                    altar = "crimson";
                }
                return "If you smash a " + altar + " altar with your Pwnhammer, new treasures will appear underground... everything comes at a cost, however.";
            case 1:
                if(NPC.downedMechBossAny)
                {
                    goto case -1;
                }
                return "You can gather souls from monsters underground amidst darkness or light. You can also find more high in the skies. Souls make great crafting materials.";
            case 2:
                if(NPC.downedMechBossAny)
                {
                    goto case -1;
                }
                return "You better get the best gear as soon as you can; it is said powerful machines may attack at night.";
            case 3:
                if(NPC.downedMechBossAny)
                {
                    goto case -1;
                }
                return "The machines will stop attacking when you defeat them; if you don't want to wait for them, you can summon them with items made from iron and souls.";
            case 4:
                if(NPC.downedMechBossAny)
                {
                    goto case -1;
                }
                string anvil = "a Mythril or Orichalcum Anvil";
                if(WorldGen.oreTier2 == 108)
                {
                    anvil = "a Mythril Anvil";
                }
                if(WorldGen.oreTier2 == 222)
                {
                    anvil = "an Orichalcum Anvil";
                }
                return "You will need " + anvil + " in order to craft powerful equipment and rare items.";
            case 5:
                if(NPC.downedMechBossAny)
                {
                    goto case -1;
                }
                string forge = "an Adamantite or Titanium Forge";
                if(WorldGen.oreTier3 == 111)
                {
                    forge = "an Adamantite Forge";
                }
                if(WorldGen.oreTier3 == 223)
                {
                    forge = "a Titanium Forge";
                }
                return "You will need " + forge + " in order to forge the best ores into bars. Keep in mind you will need a Hellforge and rare ores to craft " + forge + ".";
            case 6:
                if(NPC.savedWizard)
                {
                    goto case -1;
                }
                return "I've heard that a wizard is lost underground; he may join us if you rescue him.";
            case 7:
                if(NPC.downedMechBossAny)
                {
                    goto case -1;
                }
                return "Defeating one of the mechanical giants may get the attention of the Steampunker.";
            case 8:
                if(WorldGen.altarCount <= 0 || NPC.downedPirates)
                {
                    goto case -1;
                }
                return "There's been reports of a band of pirates nearby; be careful.";
            case 9:
                if(NPC.AnyNPCs(160))
                {
                    goto case -1;
                }
                return "If you create a mushroom biome aboveground and build a house there, a living mushroom might arrive!... I'm not joking!";
            case 10:
                if(!NPC.downedMechBossAny || Bluemagic.eclipsePassed)
                {
                    goto case -1;
                }
                return "If a solar eclipse happens, it is your chance to collect rare and powerful equipment from the unique monsters.";
            case 11:
                if(Main.localPlayer.statLifeMax > 400)
                {
                    goto case -1;
                }
                return "You can find plants called Life Fruits growing rarely in the jungle. They can increase your health even more!";
            case 12:
                if(!NPC.downedMechBossAny || (NPC.downedMechBoss1 && NPC.downedMechBoss2 && NPC.downedMechBoss3))
                {
                    goto case -1;
                }
                return "There are a total of three great machines that must be defeated.";
            case 13:
                if(!NPC.downedMechBoss1 || !NPC.downedMechBoss2 || !NPC.downedMechBoss3 || NPC.downedPlantBoss)
                {
                    goto case -1;
                }
                return "In the jungle, sometimes you will see a pink bulb growing. If you break it, a terrible monster will become enraged at you.";
            case 14:
                if(!NPC.downedMechBoss1 || !NPC.downedMechBoss2 || !NPC.downedMechBoss3 || NPC.downedPlantBoss)
                {
                    goto case -1;
                }
                return "You may have noticed some green ore called chlorophyte growing in the jungle. You will need to craft a pickaxe from the three giant machines in order to mine it.";
            case 15:
                if(!NPC.downedPlantBoss || Bluemagic.downedAbomination > 0)
                {
                    goto case -1;
                }
                return "Chlorophyte is like no other ore. It can grow and spread to nearby mud as long as it is underground and there is not too much of it around.";
            case 16:
                if(!NPC.downedPlantBoss || Bluemagic.downedAbomination > 0)
                {
                    goto case -1;
                }
                return "You can use the temple key dropped by Plantera and a key mold to craft keys that unlock special chests in the dungeon. These chests contain unique weapons.";
            case 17:
                if(!NPC.downedPlantBoss)
                {
                    goto case -1;
                }
                return "Key molds are dropped very rarely by monsters in the ice, jungle, " + dark + ", and hallow. If you are really lucky, you may have already found one.";
            case 18:
                if(!NPC.downedPlantBoss || NPC.downedGolemBoss)
                {
                    goto case -1;
                }
                return "You can use the temple key dropped by Plantera to open the temple in the underground jungle. This temple is inhabited by monsters called Lizahrds.";
            case 19:
                if(!NPC.downedPlantBoss || NPC.downedGolemBoss)
                {
                    goto case -1;
                }
                return "In the jungle temple, you can use a Power Cell on the altar in the end room to summon a powerful golem.";
            case 20:
                if(!NPC.downedGolemBoss || Bluemagic.downedAbomination > 0)
                {
                    goto case -1;
                }
                return "You can use the beetle husks dropped by Golem to craft some powerful melee armor.";
            case 21:
                if(!NPC.downedPlantBoss || !NPC.AnyNPCs(160) || Bluemagic.downedAbomination > 0)
                {
                    goto case -1;
                }
                return "You can use the autohammer sold by " + NPC.firstNPCName(160) + " to craft shroomite bars. These are materials for some powerful ranged armor.";
            case 22:
                if(!NPC.downedPlantBoss || Bluemagic.downedPhantom)
                {
                    goto case -1;
                }
                return "You hear the screams echoing from the dungeon? There are some very powerful monsters there now, but they will also give very powerful weapons!";
            case 23:
                if(!NPC.downedPlantBoss || Bluemagic.downedAbomination > 0)
                {
                    goto case -1;
                }
                return "Sometimes monsters in the dungeon will drop ectoplasm, which can be used to craft some powerful magic armor.";
            case 24:
                if(!NPC.downedPlantBoss || Bluemagic.pumpkinMoonPassed || Bluemagic.downedAbomination > 0)
                {
                    goto case -1;
                }
                return "You can use pumpkins from the dryad and ectoplasm to craft a pumpkin moon medallion. When used during the night, it will summon extremely powerful monsters that drop very powerful equipment.";
            case 25:
                if(!NPC.downedPlantBoss || Bluemagic.pumpkinMoonPassed)
                {
                    goto case -1;
                }
                return "Monsters from the pumpkin moon drop more loot the more waves you have completed. Can you reach the final wave?";
            case 26:
                if(!NPC.downedPlantBoss || Bluemagic.snowMoonPassed || Bluemagic.downedAbomination > 0)
                {
                    goto case -1;
                }
                return "You can use silk and ectoplasm to craft a naughty present. When used during the night, it will summon extremely powerful monsters that drop extremely powerful equipment.";
            case 27:
                if(!NPC.downedPlantBoss || Bluemagic.snowMoonPassed)
                {
                    goto case -1;
                }
                return "Monsters from the frost moon drop more loot the more waves you have completed. Can you reach wave 15, or maybe even further?";
            case 28:
                if(!NPC.downedPlantBoss || NPC.downedFishron)
                {
                    goto case -1;
                }
                return "You may sometimes find a truffle worm in an underground glowing mushroom area. Catch it with a bug net, but if you're too slow, it will run away!";
            case 29:
                if(!NPC.downedPlantBoss || NPC.downedFishron)
                {
                    goto case -1;
                }
                return "If you go fishing with the truffle worm in the ocean, you will catch a very powerful monster.";
            case 30:
                if(!NPC.downedPlantBoss || Bluemagic.downedPhantom)
                {
                    goto case -1;
                }
                return "You can use ectoplasm to craft an special emblem. When used in the dungeon, it will awaken a very powerful spirit.";
            case 31:
                if(!Bluemagic.pumpkinMoonPassed || !Bluemagic.snowMoonPassed || !NPC.downedFishron || !Bluemagic.downedPhantom || Bluemagic.downedAbomination > 0)
                {
                    goto case -1;
                }
                return "You can mix together items from powerful monsters of many elements to create something that can summon a hideous beast in the underworld.";
            case 32:
                int count = 0;
                for(int k = 0; k < 200; k++)
                {
                    if(Main.npc[k].active && Main.npc[k].townNPC && Main.npc[k].type != 142)
                    {
                        count++;
                    }
                }
                if(count >= 21)
                {
                    goto case -1;
                }
                return "I feel that there are still some NPCs that may want to join us. Either build more houses or follow my prior advice.";
            default:
                if(!NPC.downedMechBossAny)
                {
                    goto case -1;
                }
                return "You can use the clentaminator and green solution sold by the Steampunker to rid Terraria of the " + dark + ".";
            case -1:
                Bluemagic.helpText++;
                Bluemagic.helpText %= 34;
                break;
            }
        }
    }
}}