using System;
using Microsoft.Xna.Framework;
using Terraria;
using TAPI;

namespace Bluemagic.NPCs {
public class NightmareFuel : Dropper
{
    public NightmareFuel() : base(s: 4f, sh: "Bluemagic:NightmareDrop") {}

    public override void SelectFrame(int frameSize)
    {
        npc.frameCounter += 1.0;
        if(npc.frameCounter >= 10.0)
        {
            npc.frame.Y += frameSize;
            if(npc.frame.Y >= Main.npcFrameCount[npc.type] * frameSize)
            {
                npc.frame.Y = 0;
            }
            npc.frameCounter = 0.0;
        }
    }

    public override bool CanSpawn(int x, int y, int type, Player spawnedOn)
    {
        int tile = (int)Main.tile[x, y].type;
        int wall = (int)Main.tile[x, y].wall;
        return Main.eclipse && Main.dayTime && y < Main.worldSurface && Bluemagic.downedAbomination > 0 && Main.rand.Next(5) <= 1;
    }

    public override void HitEffect(int hitDirection, double damage, bool isDead)
    {
        if(isDead)
        {
            for(int k = 0; k < 20; k++)
            {
                int dust = Dust.NewDust(npc.position, npc.width, npc.height, 54, 0f, 0f, 50, default(Color), 1.5f);
                Main.dust[dust].velocity *= 2f;
                Main.dust[dust].noGravity = true;
            }
            int gore = Gore.NewGore(new Vector2(npc.position.X, npc.position.Y - 10f), new Vector2((float)hitDirection, 0f), 99, npc.scale);
            Main.gore[gore].velocity *= 0.3f;
            gore = Gore.NewGore(new Vector2(npc.position.X, npc.position.Y + (float)(npc.height / 2) - 15f), new Vector2((float)hitDirection, 0f), 99, npc.scale);
            Main.gore[gore].velocity *= 0.3f;
            gore = Gore.NewGore(new Vector2(npc.position.X, npc.position.Y + (float)npc.height - 20f), new Vector2((float)hitDirection, 0f), 99, npc.scale);
            Main.gore[gore].velocity *= 0.3f;
        }
        else
        {
            for(int k = 0; k < damage / npc.lifeMax * 50.0; k++)
            {
                int dust = Dust.NewDust(npc.position, npc.width, npc.height, 54, 0f, 0f, 50, default(Color), 1.5f);
                Main.dust[dust].velocity *= 2f;
                Main.dust[dust].noGravity = true;
            }
        }
    }

    public override void SetupLootRules(NPC unused)
    {
        bool flag = LootRule.settingUp;
        LootRule.settingUp = true;
        LootRule[] rules = new LootRule[1];
        rules[0] = new LootRule(null).Chance(0.005).Item("Bluemagic:NightmareFuelBanner");
        LootRule.AddFor("Bluemagic:NightmareFuel", rules);
        LootRule.settingUp = flag;
    }
}}