using System;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Terraria;
using TAPI;

namespace Bluemagic.NPCs {
public class Puppet : Hover
{
    public Puppet() : base(3f, 0.1f, 2f, 0.1f) {}

    public override void SelectFrame(int frameSize)
    {
        npc.rotation = npc.velocity.X / 10f;
        npc.spriteDirection = npc.direction;
        npc.gfxOffY = 8;
        npc.alpha = Collision.SolidCollision(npc.position, npc.width, npc.height) ? 100 : 0;
    }

    public override bool CanSpawn(int x, int y, int type, Player spawnedOn)
    {
        int tile = (int)Main.tile[x, y].type;
        int wall = (int)Main.tile[x, y].wall;
        return Main.eclipse && Main.dayTime && y < Main.worldSurface && Bluemagic.downedAbomination > 0 && Main.rand.Next(2) == 0;
    }

    public override void DealtPlayer(Player player, int hitDir, int dmgDealt, bool crit)
    {
        if(Main.rand.Next(5) == 0)
        {
            player.AddBuff("Bluemagic:Puppet", 300, true);
        }
    }

    public override void HitEffect(int hitDirection, double damage, bool isDead)
    {
        double num = damage / npc.lifeMax * 20.0;
        if(isDead)
        {
            for(int k = 0; k < 10; k++)
            {
                int dust = BluemagicDust.CreateSmoke(npc.position - new Vector2(0f, 36f), npc.width, 36, Color.Black);
                Main.dust[dust].velocity *= 0.5f;
                Main.dust[dust].scale = 2f;
            }
            if(num > 20.0)
            {
                num = 20.0;
            }
            Gore.NewGore(npc.position, npc.velocity, "Bluemagic:PuppetHead", 1f);
            Gore.NewGore(npc.position, npc.velocity, "Bluemagic:PuppetBody", 1f);
            Gore.NewGore(npc.position, npc.velocity, "Bluemagic:PuppetArm", 1f);
            Gore.NewGore(npc.position, npc.velocity, "Bluemagic:PuppetLeg", 1f);
        }
        for(int k = 0; k < num; k++)
        {
            Dust.NewDust(npc.position, npc.width, npc.height, 7);
        }
    }

    public override void SetupLootRules(NPC unused)
    {
        bool flag = LootRule.settingUp;
        LootRule.settingUp = true;
        LootRule[] rules = new LootRule[2];
        rules[0] = new LootRule(null).Chance(0.005).Item("Bluemagic:PuppetBanner");
        rules[1] = new LootRule(null).Stack(4, 5).Item("Wood");
        LootRule.AddFor("Bluemagic:Puppet", rules);
        LootRule.settingUp = flag;
    }
}}