using System;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Terraria;
using TAPI;

namespace Bluemagic.NPCs {
public class NightSlime : ModNPC
{
    private int sparkleFrame = 0;
    private static Random sparkleRand = new Random();

    public override void AI()
    {
        Player player = Main.player[npc.target];
        if (npc.type == Bluemagic.NPCID("NightSlime") && Main.rand.Next(30) == 0)
        {
            BluemagicDust.CreateSparkle(npc.position, npc.width, npc.height);
        }
        npc.ai[0] += 2f;
        if (npc.localAI[0] > 0f)
        {
            npc.localAI[0] -= 1f;
        }
        Vector2 center = new Vector2(npc.position.X + (float)npc.width * 0.5f, npc.position.Y + (float)npc.height * 0.5f);
        float shootToX = player.position.X + (float)player.width * 0.5f - center.X;
        float shootToY = player.position.Y - center.Y;
        float distance = (float)System.Math.Sqrt((double)(shootToX * shootToX + shootToY * shootToY));
        if (distance < 480f && Collision.CanHit(npc.position, npc.width, npc.height, player.position, player.width, player.height))
        {
            if (npc.velocity.Y == 0f)
            {
                npc.velocity.X *= 0.9f;
            }
            if (Main.netMode != 1 && npc.localAI[0] == 0f)
            {
                string type = "Bluemagic:StarGel";
                int damage = 35;
                if(npc.type == Bluemagic.NPCID("DirtySlime"))
                {
                    type = "Bluemagic:DirtGel";
                    damage = 45;
                }
                distance = 3f / distance;
                shootToX *= distance;
                shootToY *= distance;
                npc.localAI[0] = 120f;
                Projectile.NewProjectile(center.X, center.Y, shootToX, shootToY, ProjDef.byName[type].type, damage, 0f, Main.myPlayer, 0f, 0f);
            }
        }
    }

    public override bool CanSpawn(int x, int y, int type, Player spawnedOn)
    {
        int tile = (int)Main.tile[x, y].type;
        int wall = (int)Main.tile[x, y].wall;
        if(!Bluemagic.NoBiomeNormalSpawn(spawnedOn, tile, wall, y) || Bluemagic.downedAbomination <= 0)
        {
            return false;
        }
        if(type == Bluemagic.NPCID("NightSlime"))
        {
            return y <= Main.worldSurface && !Main.dayTime && Main.rand.Next(2) == 0;
        }
        if(type == Bluemagic.NPCID("DirtySlime"))
        {
            return y > Main.worldSurface && y < Main.hellLayer && Main.rand.Next(2) == 0;
        }
        return false;
    }

    public override void DealtPlayer(Player player, int hitDir, int dmgDealt, bool crit)
    {
        int chance = 10;
        int time = 300;
        if(npc.type == Bluemagic.NPCID("DirtySlime"))
        {
            chance = 9;
            time = 360;
        }
        if(Main.rand.Next(chance) == 0)
        {
            player.AddBuff("Bluemagic:Liquified", time, true);
            BluemagicNet.SendTransform(player.whoAmI, 1, time);
        }
    }

    public override void HitEffect(int hitDirection, double damage, bool isDead)
    {
        Color color = npc.color;
        if(npc.type == Bluemagic.NPCID("DirtySlime"))
        {
            color = new Color(178, 152, 71);
        }
        if(isDead)
        {
            for(int k = 0; k < 50; k++)
            {
                Dust.NewDust(npc.position, npc.width, npc.height, 4, (float)(2 * hitDirection), -2f, npc.alpha, color, 1f);
                if(k % 5 == 0 && npc.type == Bluemagic.NPCID("NightSlime"))
                {
                    BluemagicDust.CreateSparkle(npc.position, npc.width, npc.height);
                }
            }
        }
        else
        {
            int count = 0;
            while(count < damage / npc.lifeMax * 100)
            {
                Dust.NewDust(npc.position, npc.width, npc.height, 4, (float)hitDirection, -1f, npc.alpha, color, 1f);
                if(count % 5 == 0 && npc.type == Bluemagic.NPCID("NightSlime"))
                {
                    int dust = BluemagicDust.CreateSparkle(npc.position, npc.width, npc.height);
                    Main.dust[dust].velocity.X += hitDirection * 3;
                }
                count++;
            }
        }
    }

    public override void SelectFrame(int frameSize)
    {
        if(npc.type == Bluemagic.NPCID("NightSlime"))
        {
            sparkleFrame = (sparkleRand.Next(2) * frameSize + sparkleFrame) % (3 * frameSize);
        }
    }

    public override void SetupLootRules(NPC type)
    {
        bool flag = LootRule.settingUp;
        LootRule.settingUp = true;
        LootRule[] rules = new LootRule[4];
        rules[0] = new LootRule(null).Item("Gel").Stack(4, 6);
        rules[1] = new LootRule(null).Chance(0.0001).Item("Slime Staff");
        rules[2] = new LootRule(null).Chance(0.5).Item("Bluemagic:SuspiciousGel");
        rules[3] = new LootRule(null).Chance(0.005).Item("Bluemagic:SuspiciousSlimeBanner");
        LootRule.AddFor(type.name, rules);
        LootRule.settingUp = flag;
    }

    public override void PostDraw(SpriteBatch sb)
    {
        if(npc.type == Bluemagic.NPCID("NightSlime"))
        {
            sb.Draw(modBase.textures["NPCs/Night/NightSlimeSparkle"], npc.position - Main.screenPosition, new Rectangle(0, sparkleFrame, npc.width, npc.height), Color.White, 0f, Vector2.Zero, 1f / 1.25f, SpriteEffects.None, 0f);
        }
    }
}}