using System;
using Microsoft.Xna.Framework;
using Terraria;
using TAPI;

namespace Bluemagic.NPCs {
public class Eel : Worm
{
    public override void CustomBehavior()
    {
        if(Main.rand.Next(10) == 0)
        {
            BluemagicDust.CreateSpark(npc.position - new Vector2(60, 60), npc.width + 120, npc.height + 120);
            Player player = Main.player[Main.myPlayer];
            if(player.active && !player.dead && Vector2.Distance(player.Center, npc.Center) < 96f)
            {
                Projectile.NewProjectile(player.Center, Vector2.Zero, "Bluemagic:Sparks", 80, 0f, Main.myPlayer);
            }
        }
    }

    public override bool CanSpawn(int x, int y, int type, Player spawnedOn)
    {
        if(type != NPCDef.byName["Bluemagic:EelHead"].type)
        {
            return false;
        }
        int tile = (int)Main.tile[x, y].type;
        int wall = (int)Main.tile[x, y].wall;
        return (Bluemagic.NormalSpawn(y) && TileDef.tileSand[tile] && Bluemagic.NoZoneAllowWater(spawnedOn, tile, wall) && NPC.areaWater) && y < Main.rockLayer && (x < 250 || x > Main.maxTilesX - 250) && !NPC.areaTown && Bluemagic.downedAbomination > 0 && Main.rand.Next(2) == 0;
    }

    public override void DealtPlayer(Player player, int hitDir, int dmgDealt, bool crit)
    {
        if(Main.rand.Next(5) == 0)
        {
            player.AddBuff(33, 3600, true);
        }
    }

    public override void HitEffect(int hitDirection, double damage, bool isDead)
    {
        if(isDead)
        {
            for(int k = 0; k < 10; k++)
            {
                Dust.NewDust(npc.position, npc.width, npc.height, 5, 2.5f * (float)hitDirection, -2.5f, 0, default(Color), 1f);
            }
            if(head)
            {
                Gore.NewGore(npc.position, npc.velocity, "Bluemagic:EelHead", 1f);
            }
            else if(tail)
            {
                Gore.NewGore(npc.position, npc.velocity, "Bluemagic:EelTail", 1f);
            }
            else
            {
                Gore.NewGore(npc.position, npc.velocity, "Bluemagic:EelBody", 1f);
            }
        }
        else
        {
            for(int k = 0; k < damage / (double)npc.lifeMax * 50.0; k++)
            {
                Dust.NewDust(npc.position, npc.width, npc.height, 5, (float)hitDirection, -1f, 0, default(Color), 1f);
            }
        }
    }

    public override void SelectFrame(int frameSize)
    {
        npc.gfxOffY = 9;
    }

    public override void SetupLootRules(NPC type)
    {
        bool flag = LootRule.settingUp;
        LootRule.settingUp = true;
        LootRule[] rules = new LootRule[2];
        rules[0] = new LootRule(null).Chance(0.02).Item("Vitamins");
        rules[1] = new LootRule(null).Chance(0.005).Item("Bluemagic:EelBanner");
        LootRule.AddFor(type.name, rules);
        LootRule.settingUp = flag;
    }
}}