using System;
using Microsoft.Xna.Framework;
using Terraria;
using TAPI;

namespace Bluemagic.NPCs {
public class Octopus : Fish
{
    public Octopus() : base(1f, 1f, 0.05f, 0.05f, 0.95f, true, 0.5f, false) {}

    public override bool PreAI()
    {
        if(npc.localAI[0] == 0f && Main.netMode != 1)
        {
            npc.localAI[0] = 1f;
            for(int k = 0; k < 6; k++)
            {
                int proj = Projectile.NewProjectile(npc.Center, Vector2.Zero, "Bluemagic:OctopusArm", npc.damage / 2, 0, Main.myPlayer);
                if(proj == 1000)
                {
                    npc.active = false;
                    return false;
                }
                Projectiles.OctopusArm arm = Main.projectile[proj].GetSubClass<Projectiles.OctopusArm>();
                Main.projectile[proj].localAI[1] = npc.whoAmI;
                arm.width = 16f;
                arm.length = Projectiles.OctopusArm.minLength;
                arm.minAngle = (k - 0.5f) * (float)Math.PI / 3f;
                arm.maxAngle = (k + 0.5f) * (float)Math.PI / 3f;
                arm.angle = (arm.minAngle + arm.maxAngle) / 2f;
                if(Main.netMode == 2)
                {
                    BluemagicNet.SendOctopusArmData(proj);
                }
            }
        }
        return true;
    }

    public override bool CanSpawn(int x, int y, int type, Player spawnedOn)
    {
        int tile = (int)Main.tile[x, y].type;
        int wall = (int)Main.tile[x, y].wall;
        return (Bluemagic.NormalSpawn(y) && TileDef.tileSand[tile] && Bluemagic.NoZoneAllowWater(spawnedOn, tile, wall) && NPC.areaWater) && y < Main.rockLayer && (x < 250 || x > Main.maxTilesX - 250) && !NPC.areaTown && Bluemagic.downedAbomination > 0 && Main.rand.Next(2) == 0;
    }

    public override void HitEffect(int hitDirection, double damage, bool isDead)
    {
        if(isDead)
        {
            for(int k = 0; k < 20; k++)
            {
                Dust.NewDust(npc.position, npc.width, npc.height, 151, 2.5f * (float)hitDirection, -2.5f, 0, default(Color), 0.7f);
            }
            Gore.NewGore(npc.position, npc.velocity, "Bluemagic:OctopusHead", 1f);
            for(int k = 0; k < 8; k++)
            {
                Vector2 pos = npc.position + new Vector2(Main.rand.Next(npc.width - 8), Main.rand.Next(npc.height / 2));
                Gore.NewGore(pos, npc.velocity, "Bluemagic:OctopusArm", 1f);
            }
        }
        else
        {
            for(int k = 0; k < damage / npc.lifeMax * 50.0; k++)
            {
                Dust.NewDust(npc.position, npc.width, npc.height, 151, (float)hitDirection, -1f, 0, default(Color), 0.7f);
            }
        }
    }

    public override void SelectFrame(int frameSize)
    {
        npc.frame.Y = 0;
        npc.rotation = 0f;
    }

    public override void SetupLootRules(NPC unused)
    {
        bool flag = LootRule.settingUp;
        LootRule.settingUp = true;
        LootRule[] rules = new LootRule[1];
        rules[0] = new LootRule(null).Chance(0.005).Item("Bluemagic:OctopusBanner");
        LootRule.AddFor("Bluemagic:Octopus", rules);
        LootRule.settingUp = flag;
    }
}}