using System;
using Microsoft.Xna.Framework;
using Terraria;
using TAPI;

namespace Bluemagic.NPCs {
public class Pufferfish : Fish
{
    public Pufferfish() : base(1f, 1f, 0.05f, 0.05f, 0.95f, true, 0.5f, true) {}

    public override void AI()
    {
        base.AI();
        Player player = Main.player[npc.target];
        if(Main.netMode != 1)
        {
            if(npc.ai[1] < 100f)
            {
                npc.ai[1] += 1f;
            }
            if(npc.ai[1] >= 100f && Math.Abs(player.Center.X - npc.Center.X) < 576 && Math.Abs(player.Center.Y - npc.Center.Y) < 432 && Collision.CanHit(npc.position, npc.width, npc.height, player.position, player.width, player.height))
            {
                Vector2 direction = player.Center - npc.Center;
                direction += new Vector2(Main.rand.Next(-10, 11), Main.rand.Next(-10, 11));
                if(direction == Vector2.Zero)
                {
                    direction = new Vector2(npc.direction * 8f, 0f);
                }
                else
                {
                    direction.Normalize();
                    direction *= 8f;
                }
                int proj = Projectile.NewProjectile(npc.Center, direction, "Bluemagic:Needle", (npc.damage - 10) / 2, 0f, Main.myPlayer, 0f, 0f);
                Main.projectile[proj].timeLeft = 300;
                npc.ai[1] = 0f;
                npc.netUpdate = true;
                BluemagicNet.PlaySound(2, (int)npc.position.X, (int)npc.position.Y, 17);
            }
        }
    }

    public override bool CanSpawn(int x, int y, int type, Player spawnedOn)
    {
        int tile = (int)Main.tile[x, y].type;
        int wall = (int)Main.tile[x, y].wall;
        return (Bluemagic.NormalSpawn(y) && TileDef.tileSand[tile] && Bluemagic.NoZoneAllowWater(spawnedOn, tile, wall) && NPC.areaWater) && y < Main.rockLayer && (x < 250 || x > Main.maxTilesX - 250) && !NPC.areaTown && Bluemagic.downedAbomination > 0 && Main.rand.Next(2) == 0;
    }

    public override void DealtPlayer(Player player, int hitDir, int dmgDealt, bool crit)
    {
        player.AddBuff(70, 400, true);
    }

    public override void HitEffect(int hitDirection, double damage, bool isDead)
    {
        if(isDead)
        {
            for(int k = 0; k < 50; k++)
            {
                Dust.NewDust(npc.position, npc.width, npc.height, 5, 2f * hitDirection, -2f, 0, default(Color), 1f);
            }
            Gore.NewGore(npc.position, npc.velocity, "Bluemagic:Pufferfish1", 1f);
            Gore.NewGore(npc.position + new Vector2(npc.width / 2f, 0f), npc.velocity, "Bluemagic:Pufferfish2", 1f);
        }
        else
        {
            for(int k = 0; k < damage / npc.lifeMax * 50.0; k++)
            {
                Dust.NewDust(npc.position, npc.width, npc.height, 5, (float)hitDirection, -1f, 0, default(Color), 1f);
            }
        }
    }

    public override void SetupLootRules(NPC unused)
    {
        bool flag = LootRule.settingUp;
        LootRule.settingUp = true;
        LootRule[] rules = new LootRule[2];
        rules[0] = new LootRule(null).Chance(0.02).Item("Bezoar");
        rules[1] = new LootRule(null).Chance(0.005).Item("Bluemagic:PufferfishBanner");
        LootRule.AddFor("Bluemagic:Pufferfish", rules);
        LootRule.settingUp = false;
    }
}}