using System;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Terraria;
using TAPI;

namespace Bluemagic.NPCs {
public class PhantomOrb : ModNPC
{
    private bool sentLocal = false;

    public override void OnSpawn()
    {
        npc.localAI[0] = -1f;
        npc.localAI[1] = -1f;
        for(int x = 0; x < 140; x++)
        {
            npc.buffImmune[x] = true;
        }
        npc.ai[1] = 120f;
    }

    public override void AI()
    {
        npc.rotation += 0.1f;
        if(npc.rotation > 2f * (float)Math.PI)
        {
            npc.rotation -= 2f * (float)Math.PI;
        }
        if(npc.localAI[0] >= 0f)
        {
            NPC follow = Main.npc[(int)npc.localAI[0]];
            if(follow.type == NPCDef.byName["Bluemagic:Phantom"].type)
            {
                npc.position = follow.Center + new Vector2(-npc.width / 2f, follow.height / 2f);
                if(npc.ai[0] > 0f)
                {
                    npc.position.X -= follow.width * 1.5f;
                }
                else
                {
                    npc.position.X += follow.width * 1.5f;
                }
            }
            else
            {
                npc.position = follow.Center - new Vector2(npc.width / 2, npc.height / 2);
            }
        }
        if(Main.netMode == 2 && !sentLocal)
        {
            BluemagicNet.SendNPCLocalData(npc.whoAmI);
            sentLocal = true;
        }
        npc.ai[1] -= 1f;
        npc.netUpdate = true;
        if(npc.ai[1] <= 0f)
        {
            if(Main.netMode != 1 && npc.ai[0] == 1f)
            {
                int blade = Projectile.NewProjectile(npc.position, Vector2.Zero, "Bluemagic:PhantomBladeBoss", npc.damage / 2, 0, Main.myPlayer, npc.localAI[0], npc.localAI[1]);
                Projectiles.DiagonalHitbox hitbox = Main.projectile[blade].GetSubClass<Projectiles.PhantomBladeBoss>();
                hitbox.length = 400;
                hitbox.width = 24;
                hitbox.angle = 1.25f * (float)Math.PI;
                Main.projectile[blade].localAI[1] = npc.localAI[0];
                if(Main.netMode == 2)
                {
                    BluemagicNet.SendProjAllData(blade);
                }
            }
            else if(Main.netMode != 1 && npc.ai[0] == -1f)
            {
                for(int k = 0; k < 4; k++)
                {
                    int wisp = Projectile.NewProjectile(npc.position, Vector2.Zero, "Bluemagic:WispBoss", npc.damage / 2, 0, Main.myPlayer, 25 * k, 0f);
                    Main.projectile[wisp].localAI[0] = npc.localAI[0];
                    Main.projectile[wisp].localAI[1] = npc.localAI[1];
                }
            }
            else if(Main.netMode != 1)
            {
                NPC paladin = Main.npc[NPC.NewNPC(npc.position, 290, 0)];
                paladin.position = npc.Center - paladin.Size / 2f;
                if(Collision.SolidCollision(paladin.position, paladin.width, paladin.height))
                {
                    Player player = Main.player[Main.npc[(int)npc.localAI[1]].target];
                    paladin.position = player.position;
                    paladin.position.X += ((float)player.width - (float)paladin.width) / 2f;
                    paladin.position.Y += (float)player.height - (float)paladin.height;
                }
                if(Main.netMode == 2)
                {
                    BluemagicNet.SendNPCData(paladin.whoAmI);
                }
            }
            npc.active = false;
        }
    }

    public override void HitEffect(int hitDirection, double damage, bool isDead)
    {
        if(isDead)
        {
            for(int x = 0; x < 50; x++)
            {
                int dust = BluemagicDust.CreatePhantomDust(npc.Center.X - 4f, npc.Center.Y - 4f);
                Main.dust[dust].velocity *= 2f;
            }
        }
    }

    public override bool PreNPCLoot()
    {
        return false;
    }

    public override bool PreDraw(SpriteBatch sb)
    {
        sb.Draw(Main.npcTexture[npc.type], npc.Center - Main.screenPosition, null, Color.White * ((255 - npc.alpha) / 255f), npc.rotation, new Vector2(npc.width / 2, npc.height / 2), 1f, SpriteEffects.None, 0f);
        return false;
    }
}}