using System;
using Microsoft.Xna.Framework;
using Terraria;
using TAPI;

namespace Bluemagic.NPCs {
public class Spitter : ModNPC
{
    private float speed;
    private float acceleration;
    private bool circles;
    private float bounce;
    private int dustType;
    private string shoot;
    private int shootTime;
    private float shootSpeed;
    public Spitter(float s = 4.3f, float a = 0.023f, bool c = true, float b = 0.7f, int d = -1, string sh = "", int shTime = 180, float shSpeed = 7f)
    {
        speed = s;
        acceleration = a;
        circles = c;
        bounce = b;
        dustType = d;
        shoot = sh;
        shootTime = shTime;
        shootSpeed = shSpeed;
    }

    public override void AI()
    {
        Player player = Main.player[npc.target];
        if (npc.target < 0 || npc.target == 255 || player.dead)
        {
            npc.TargetClosest(true);
        }
        Vector2 center = new Vector2(npc.position.X + (float)npc.width * 0.5f, npc.position.Y + (float)npc.height * 0.5f);
        float offsetX = player.position.X + (float)(player.width / 2);
        float offsetY = player.position.Y + (float)(player.height / 2);
        offsetX = (float)((int)(offsetX / 8f) * 8);
        offsetY = (float)((int)(offsetY / 8f) * 8);
        center.X = (float)((int)(center.X / 8f) * 8);
        center.Y = (float)((int)(center.Y / 8f) * 8);
        offsetX -= center.X;
        offsetY -= center.Y;
        float distance = (float)System.Math.Sqrt((double)(offsetX * offsetX + offsetY * offsetY));
        if (distance == 0f)
        {
            offsetX = npc.velocity.X;
            offsetY = npc.velocity.Y;
        }
        else
        {
            float adjust = speed / distance;
            offsetX *= adjust;
            offsetY *= adjust;
        }

        if (distance > 100f || circles)
        {
            npc.ai[0] += 1f;
            if (npc.ai[0] > 0f)
            {
                npc.velocity.Y += 0.023f;
            }
            else
            {
                npc.velocity.Y -= 0.023f;
            }
            if (npc.ai[0] < -100f || npc.ai[0] > 100f)
            {
                npc.velocity.X += 0.023f;
            }
            else
            {
                npc.velocity.X -= 0.023f;
            }
            if (npc.ai[0] > 200f)
            {
                npc.ai[0] = -200f;
            }
        }
        if (distance < 150f)
        {
            npc.velocity.X += offsetX * 0.007f;
            npc.velocity.Y += offsetY * 0.007f;
        }

        if (player.dead)
        {
            offsetX = (float)npc.direction * speed / 2f;
            offsetY = -speed / 2f;
        }
        if (npc.velocity.X < offsetX)
        {
            npc.velocity.X += acceleration;
        }
        else if (npc.velocity.X > offsetX)
        {
            npc.velocity.X -= acceleration;
        }
        if (npc.velocity.Y < offsetY)
        {
            npc.velocity.Y += acceleration;
        }
        else if (npc.velocity.Y > offsetY)
        {
            npc.velocity.Y -= acceleration;
        }
        npc.rotation = (float)System.Math.Atan2((double)offsetY, (double)offsetX) - 1.57f;

        if (npc.collideX)
        {
            npc.netUpdate = true;
            npc.velocity.X = npc.oldVelocity.X * -bounce;
            if (npc.direction == -1 && npc.velocity.X > 0f && npc.velocity.X < 2f)
            {
                npc.velocity.X = 2f;
            }
            if (npc.direction == 1 && npc.velocity.X < 0f && npc.velocity.X > -2f)
            {
                npc.velocity.X = -2f;
            }
        }
        if (npc.collideY)
        {
            npc.netUpdate = true;
            npc.velocity.Y = npc.oldVelocity.Y * -bounce;
            if (npc.velocity.Y > 0f && (double)npc.velocity.Y < 1.5)
            {
                npc.velocity.Y = 2f;
            }
            if (npc.velocity.Y < 0f && (double)npc.velocity.Y > -1.5)
            {
                npc.velocity.Y = -2f;
            }
        }
        if (dustType >= 0 && Main.rand.Next(20) == 0)
        {
            int dust = Dust.NewDust(new Vector2(npc.position.X, npc.position.Y + (float)npc.height * 0.25f), npc.width, (int)((float)npc.height * 0.5f), dustType, npc.velocity.X, 2f, 75, npc.color, npc.scale);
            Main.dust[dust].velocity.X *= 0.5f;
            Main.dust[dust].velocity.Y *= 0.1f;
        }

        if (npc.wet)
        {
            if (npc.velocity.Y > 0f)
            {
                npc.velocity.Y *= 0.95f;
            }
            npc.velocity.Y -= 0.3f;
            if (npc.velocity.Y < -2f)
            {
                npc.velocity.Y = -2f;
            }
        }

        if (Main.netMode != 1 && !shoot.Equals("") && !player.dead)
        {
            if (npc.justHit)
            {
                npc.localAI[0] = 0f;
            }
            npc.localAI[0] += 1f;
            if (npc.localAI[0] >= 180f)
            {
                if (Collision.CanHit(npc.position, npc.width, npc.height, player.position, player.width, player.height))
                {
                    Projectile.NewProjectile(npc.Center + npc.velocity, new Vector2(offsetX, offsetY) * shootSpeed / speed, shoot, npc.damage / 2, 0f, Main.myPlayer);
                }
                npc.localAI[0] = 0f;
            }
        }

        if (player.dead)
        {
            npc.velocity.Y -= acceleration * 2f;
            if (npc.timeLeft > 10)
            {
                npc.timeLeft = 10;
            }
        }
        if (((npc.velocity.X > 0f && npc.oldVelocity.X < 0f) || (npc.velocity.X < 0f && npc.oldVelocity.X > 0f) || (npc.velocity.Y > 0f && npc.oldVelocity.Y < 0f) || (npc.velocity.Y < 0f && npc.oldVelocity.Y > 0f)) && !npc.justHit)
        {
            npc.netUpdate = true;
        }
    }
}}