using System;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Terraria;
using TAPI;

namespace Bluemagic.Projectiles {
public class DiagonalHitbox : CustomHitbox
{
    public Vector2 origin
    {
        get
        {
            return projectile.position;
        }
        set
        {
            projectile.position = value;
        }
    }
    public float width
    {
        get
        {
            return projectile.ai[0];
        }
        set
        {
            projectile.ai[0] = value;
        }
    }
    public float length
    {
        get
        {
            return projectile.ai[1];
        }
        set
        {
            projectile.ai[1] = value;
        }
    }
    public float angle
    {
        get
        {
            return projectile.localAI[0];
        }
        set
        {
            projectile.localAI[0] = value;
        }
    }
    private Texture2D tip = null;
    public DiagonalHitbox(string t)
    {
        if(!Main.dedServ)
        {
            tip = Bluemagic.instance.textures[t];
        }
    }

    public DiagonalHitbox() {}

    public override bool Collides(CodableEntity entity)
    {
        while(angle < 0f)
        {
            angle += 2f * (float)Math.PI;
        }
        angle %= 2f * (float)Math.PI;
        if(angle % ((float)Math.PI / 2f) == 0f)
        {
            float top = origin.Y;
            float left = origin.X;
            float rectWidth = length;
            float rectHeight = length;
            if(angle % (float)Math.PI == 0f)
            {
                top -= width / 2f;
                rectHeight = width;
            }
            else
            {
                left -= width / 2f;
                rectWidth = width;
            }
            if(angle == (float)Math.PI)
            {
                left -= length;
            }
            if(angle == (float)Math.PI * 1.5f)
            {
                top -= length;
            }
            return new Rectangle((int)left, (int)top, (int)rectWidth, (int)rectHeight).Intersects(entity.Hitbox);
        }
        float pAngle = angle + (float)Math.PI / 2f;
        Vector2 direction = new Vector2((float)Math.Cos(angle), (float)Math.Sin(angle));
        Vector2 normal = new Vector2((float)Math.Cos(pAngle), (float)Math.Sin(pAngle));
        direction *= length;
        normal *= width / 2f;
        Line topLeft = new Line(origin, pAngle);
        Line topRight = new Line(origin - normal, angle);
        Line bottomLeft = new Line(origin + normal, angle);
        Line bottomRight = new Line(origin + direction, pAngle);
        for(float k = (float)Math.PI / 2f; k < angle; k += (float)Math.PI / 2f)
        {
            Line temp = topLeft;
            topLeft = bottomLeft;
            bottomLeft = bottomRight;
            bottomRight = topRight;
            topRight = temp;
        }
        Vector2 entTopLeft = entity.position;
        Vector2 entTopRight = entity.position + new Vector2(entity.width, 0f);
        Vector2 entBottomLeft = entity.position + new Vector2(0f, entity.height);
        Vector2 entBottomRight = entity.position + entity.Size;
        return topLeft.f(entBottomRight.X) < entBottomRight.Y && topRight.f(entBottomLeft.X) < entBottomLeft.Y && bottomLeft.f(entTopRight.X) > entTopRight.Y && bottomRight.f(entTopLeft.X) > entTopLeft.Y;
    }

    public override int AttackDirection(CodableEntity target)
    {
        angle %= 2f * (float)Math.PI;
        if(angle % (float)Math.PI == 0f)
        {
            return -entity.direction;
        }
        if(angle % (float)Math.PI / 2f == 0f)
        {
            return target.Center.X < origin.X ? -1 : 1;
        }
        float yOffset = target.Center.Y - origin.Y;
        float x = origin.X + yOffset / (float)Math.Tan(angle);
        return target.Center.X < x ? -1 : 1;
    }

    public override bool PreDraw(SpriteBatch sb)
    {
        Texture2D unit = Main.projectileTexture[projectile.type];
        int unitLength = unit.Width;
        int numUnits = (int)Math.Ceiling(length / unitLength);
        float increment = 0f;
        if(numUnits > 1)
        {
            increment = (length - unitLength) / (numUnits - 1);
        }
        Vector2 direction = new Vector2((float)Math.Cos(angle), (float)Math.Sin(angle));
        SpriteEffects effects = SpriteEffects.None;
        if(projectile.spriteDirection == -1)
        {
            effects = SpriteEffects.FlipVertically;
        }
        for(int k = 1; k <= numUnits; k++)
        {
            Texture2D image = unit;
            if(k == numUnits && tip != null)
            {
                image = tip;
            }
            Vector2 pos = origin + direction * (increment * (k - 1) + unitLength / 2f);
            Color color = Lighting.GetColor((int)(pos.X / 16f), (int)(pos.Y / 16f));
            sb.Draw(image, pos - Main.screenPosition, null, GetAlpha(color), angle, new Vector2(unit.Width / 2, unit.Height / 2), 1f, effects, 0f);
        }
        return false;
    }

    public virtual Color GetAlpha(Color color)
    {
        return color * ((255 - projectile.alpha) / 255f);
    }
}}