using System;
using Microsoft.Xna.Framework;
using LitJson;
using Terraria;
using TAPI;

namespace Bluemagic.Projectiles {
public class HoverShooter : Minion
{
    private float idleAccel = 0.05f;
    private float spacingMult = 1f;
    private float viewDist = 400f;
    private float chaseDist = 200f;
    private float chaseAccel = 6f;
    private float inertia = 40f;
    private float shootCool = 90f;
    private float shootSpeed;
    private string shoot;
    public override void OnSpawn()
    {
        JsonData data = projectile.def.json;
        Bluemagic.jsonFloat(data, "idleAccel", ref idleAccel);
        Bluemagic.jsonFloat(data, "spacingMult", ref spacingMult);
        Bluemagic.jsonFloat(data, "viewDist", ref viewDist);
        Bluemagic.jsonFloat(data, "chaseDist", ref chaseDist);
        Bluemagic.jsonFloat(data, "chaseAccel", ref chaseAccel);
        Bluemagic.jsonFloat(data, "inertia", ref inertia);
        Bluemagic.jsonFloat(data, "shootCool", ref shootCool);
        shootSpeed = (float)data["shootSpeed"];
        shoot = (string)data["shoot"];
    }

    public virtual void CreateDust() {}

    public virtual void SelectFrame() {}

    public override void AI()
    {
        Player player = Main.player[projectile.owner];
        float spacing = (float)projectile.width * spacingMult;
        for (int k = 0; k < 1000; k++)
        {
            Projectile otherProj = Main.projectile[k];
            if (k != projectile.whoAmI && otherProj.active && otherProj.owner == projectile.owner && otherProj.type == projectile.type && System.Math.Abs(projectile.position.X - otherProj.position.X) + System.Math.Abs(projectile.position.Y - otherProj.position.Y) < spacing)
            {
                if (projectile.position.X < Main.projectile[k].position.X)
                {
                    projectile.velocity.X -= idleAccel;
                }
                else
                {
                    projectile.velocity.X += idleAccel;
                }
                if (projectile.position.Y < Main.projectile[k].position.Y)
                {
                    projectile.velocity.Y -= idleAccel;
                }
                else
                {
                    projectile.velocity.Y += idleAccel;
                }
            }
        }

        Vector2 targetPos = projectile.position;
        float targetDist = viewDist;
        bool target = false;
        projectile.tileCollide = true;
        for (int k = 0; k < 200; k++)
        {
            NPC npc = Main.npc[k];
            if (npc.active && !npc.dontTakeDamage && !npc.friendly && npc.lifeMax > 5)
            {
                float distance = Vector2.Distance(npc.center(), projectile.center());
                if (distance < targetDist && Collision.CanHitLine(projectile.position, projectile.width, projectile.height, npc.position, npc.width, npc.height))
                {
                    targetDist = distance;
                    targetPos = npc.center();
                    target = true;
                }
            }
        }

        if (Vector2.Distance(player.Center, projectile.center()) > (target ? 1000f : 500f))
        {
            projectile.ai[0] = 1f;
            projectile.netUpdate = true;
        }
        if (projectile.ai[0] == 1f)
        {
            projectile.tileCollide = false;
        }
        if (target && projectile.ai[0] == 0f)
        {
            Vector2 direction = targetPos - projectile.center();
            if (direction.Length() > chaseDist)
            {
                direction.Normalize();
                projectile.velocity = (projectile.velocity * inertia + direction * chaseAccel) / (inertia + 1);
            }
            else
            {
                projectile.velocity *= (float)Math.Pow(0.97, 40.0 / inertia);
            }
        }
        else
        {
            if (!Collision.CanHitLine(projectile.center(), 1, 1, player.Center, 1, 1))
            {
                projectile.ai[0] = 1f;
            }
            float speed = 6f;
            if (projectile.ai[0] == 1f)
            {
                speed = 15f;
            }
            Vector2 center = projectile.center();
            Vector2 direction = player.Center - center;
            projectile.ai[1] = 3600f;
            projectile.netUpdate = true;
            int num = 1;
            for (int k = 0; k < projectile.whoAmI; k++)
            {
                if (Main.projectile[k].active && Main.projectile[k].owner == projectile.owner && Main.projectile[k].type == projectile.type)
                {
                    num++;
                }
            }
            direction.X -= (float)((10 + num * 40) * player.direction);
            direction.Y -= 48f;
            float distanceTo = direction.Length();
            if (distanceTo > 200f && speed < 9f)
            {
                speed = 9f;
            }
            if (distanceTo < 100f && projectile.ai[0] == 1f && !Collision.SolidCollision(projectile.position, projectile.width, projectile.height))
            {
                projectile.ai[0] = 0f;
                projectile.netUpdate = true;
            }
            if (distanceTo > 2000f)
            {
                projectile.Center = player.Center;
            }
            if (distanceTo > 48f)
            {
                direction.Normalize();
                direction *= speed;
                float temp = inertia / 2f;
                projectile.velocity = (projectile.velocity * temp + direction) / (temp + 1);
            }
            else
            {
                projectile.direction = Main.player[projectile.owner].direction;
                projectile.velocity *= (float)Math.Pow(0.9, 40.0 / inertia);
            }
        }

        projectile.rotation = projectile.velocity.X * 0.05f;
        SelectFrame();
        CreateDust();
        if (projectile.velocity.X > 0f)
        {
            projectile.spriteDirection = (projectile.direction = -1);
        }
        else if (projectile.velocity.X < 0f)
        {
            projectile.spriteDirection = (projectile.direction = 1);
        }

        if(projectile.ai[1] > 0f)
        {
            projectile.ai[1] += 1f;
            if(Main.rand.Next(3) == 0)
            {
                projectile.ai[1] += 1f;
            }
        }
        if(projectile.ai[1] > shootCool)
        {
            projectile.ai[1] = 0f;
            projectile.netUpdate = true;
        }
        if (projectile.ai[0] == 0f)
        {
            if (target)
            {
                if ((targetPos - projectile.center()).X > 0f)
                {
                    projectile.spriteDirection = (projectile.direction = -1);
                }
                else if ((targetPos - projectile.center()).X < 0f)
                {
                    projectile.spriteDirection = (projectile.direction = 1);
                }
                if (projectile.ai[1] == 0f)
                {
                    projectile.ai[1] = 1f;
                    if (Main.myPlayer == projectile.owner)
                    {
                        Vector2 shootVel = targetPos - projectile.center();
                        shootVel.Normalize();
                        shootVel *= shootSpeed;
                        int proj = Projectile.NewProjectile(projectile.center().X, projectile.center().Y, shootVel.X, shootVel.Y, shoot, projectile.damage, 0f, Main.myPlayer, 0f, 0f);
                        Main.projectile[proj].timeLeft = 300;
                        Main.projectile[proj].netUpdate = true;
                        projectile.netUpdate = true;
                    }
                }
            }
        }
    }
}}