using System;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Terraria;
using TAPI;

namespace Bluemagic.Projectiles {
public class OctopusArm : DiagonalHitbox
{
    public float minAngle = 0f;
    public float maxAngle = 0f;
    public float angleSpeed = 0f;
    public float lengthSpeed = 0f;
    private int netUpdateCounter = 0;
    private const float maxAngleSpeed = 0.01f;
    private const float angleBuffer = (float)Math.PI / 12f;
    public const float minLength = 80f;
    private const float maxLength = 400f;
    private const float maxLengthSpeed = 1f;

    public OctopusArm() : base("Projectiles/Ocean/OctopusArmTip") {}

    public override void AI()
    {
        NPC npc = Main.npc[(int)projectile.localAI[1]];
        if(!npc.active || npc.type != NPCDef.byName["Bluemagic:Octopus"].type)
        {
            return;
        }
        projectile.timeLeft = 2;
        Player player = Main.player[npc.target];
        origin = npc.Center;
        Vector2 offset = player.Center - origin;
        float distance = offset.Length() + 32f;

        Angle currAngle = new Angle(angle);
        Angle angleToPlayer = new Angle((float)Math.Atan2(offset.Y, offset.X));
        Angle min = new Angle(minAngle);
        Angle max = new Angle(maxAngle);
        Angle limit = new Angle((minAngle + maxAngle) / 2f);
        if(limit.Between(min, max))
        {
            limit = limit.Opposite();
        }
        Angle buffer = new Angle(angleBuffer);

        if(angleToPlayer.Between(min - buffer, max + buffer))
        {
            if(currAngle.Between(max, limit))
            {
                angleSpeed -= maxAngleSpeed / 10f;
            }
            else if(currAngle.Between(limit, min))
            {
                angleSpeed += maxAngleSpeed / 10f;
            }
            else if(currAngle.ClockwiseFrom(angleToPlayer))
            {
                angleSpeed += maxAngleSpeed / 10f;
            }
            else
            {
                angleSpeed -= maxAngleSpeed / 10f;
            }

            if(length > maxLength)
            {
                lengthSpeed -= maxLengthSpeed / 10f;
            }
            else if(length < minLength)
            {
                lengthSpeed += maxLengthSpeed / 10f;
            }
            else if(distance > length)
            {
                lengthSpeed += maxLengthSpeed / 10f;
            }
            else if(distance < length)
            {
                lengthSpeed -= maxLengthSpeed / 10f;
            }
        }
        else
        {
            if(currAngle.Between(max, limit))
            {
                angleSpeed -= maxAngleSpeed / 10f;
            }
            else if(currAngle.Between(limit, min))
            {
                angleSpeed += maxAngleSpeed / 10f;
            }
            else if(angleSpeed > 0f)
            {
                angleSpeed += maxAngleSpeed / 20f;
            }
            else if(angleSpeed < 0f)
            {
                angleSpeed -= maxAngleSpeed / 20f;
            }
            else
            {
                angleSpeed = maxAngleSpeed / 20f;
            }

            if(length > minLength)
            {
                lengthSpeed -= maxLengthSpeed / 10f;
            }
            else
            {
                lengthSpeed += maxLengthSpeed / 10f;
            }
        }

        if(angleSpeed > maxAngleSpeed)
        {
            angleSpeed = maxAngleSpeed;
        }
        else if(angleSpeed < -maxAngleSpeed)
        {
            angleSpeed = -maxAngleSpeed;
        }
        if(lengthSpeed > maxLengthSpeed)
        {
            lengthSpeed = maxLengthSpeed;
        }
        else if(lengthSpeed < -maxLengthSpeed)
        {
            lengthSpeed = -maxLengthSpeed;
        }
        angle += angleSpeed;
        length += lengthSpeed;

        if(Main.netMode == 2)
        {
            netUpdateCounter++;
            if(netUpdateCounter >= 300)
            {
                BluemagicNet.SendOctopusArmData(projectile.whoAmI);
                netUpdateCounter = 0;
            }
        }
    }

    public override void PostDraw(SpriteBatch sb)
    {
        NPC npc = Main.npc[(int)projectile.localAI[1]];
        sb.Draw(Main.npcTexture[npc.type], npc.position - Main.screenPosition, null, Lighting.GetColor((int)(npc.Center.X / 16f), (int)(npc.Center.Y / 16f)), npc.rotation, Vector2.Zero, 1f, SpriteEffects.None, 0f);
    }
}}