using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Terraria;
using Terraria.DataStructures;
using TAPI;

namespace Bluemagic.Tiles {
public class WirelessTransceiver : ModTileType
{
    private static bool destroying = false;
    public static List<Point16> positions = new List<Point16>();

    public override bool CanPlace(int x, int y)
    {
        for(int j = x; j < x + 2; j++)
        {
            for(int k = y; k > y - 2; k--)
            {
                if(Main.tile[j, k] != null && Main.tile[j, k].active())
                {
                    return false;
                }
            }
        }
        bool? flag = TileDef.CheckPlacementCondition("flatGround", x, y, 2, 1);
        return flag.HasValue && flag.Value;
    }

    public override void PlaceTile(int x, int y)
    {
        SetTile(x, y, 0, 36);
        SetTile(x + 1, y, 18, 36);
        SetTile(x, y - 1, 0, 0);
        SetTile(x + 1, y - 1, 18, 0);
        positions.Add(new Point16(x, y - 1));
    }

    private static void SetTile(int x, int y, short frameX, short frameY)
    {
        if(Main.tile[x, y] == null)
        {
            Main.tile[x, y] = new Tile();
        }
        Main.tile[x, y].active(true);
        Main.tile[x, y].type = TileDef.byName["Bluemagic:WirelessTransceiver"];
        Main.tile[x, y].frameX = frameX;
        Main.tile[x, y].frameY = frameY;
        if(Main.netMode == 2)
        {
            NetMessage.SendTileSquare(-1, x, y, 1);
        }
        else
        {
            WorldGen.SquareTileFrame(x, y, true);
        }
    }

    public override void Kill(int x, int y, bool fail, bool effectsOnly, bool noItem)
    {
        if(!destroying)
        {
            destroying = true;
            GetTopLeft(ref x, ref y);
            for(int j = x; j < x + 2; j++)
            {
                for(int k = y; k < y + 2; k++)
                {
                    WorldGen.KillTile(j, k, fail, effectsOnly, noItem);
                }
            }
            positions.Remove(new Point16(x, y));
            Item.NewItem(x * 16, y * 16, 16, 16, Bluemagic.ItemID("WirelessTransceiver"), 1, false, -1, false);
            destroying = false;
        }
    }

    public override bool TileFrame(int x, int y)
    {
        if(Main.tile[x, y].frameY <= 18)
        {
            return false;
        }
        bool? flag = TileDef.CheckPlacementCondition("flatGround", x, y, 1, 1);
        if(!flag.HasValue || !flag.Value)
        {
            WorldGen.KillTile(x, y);
            return true;
        }
        return false;
    }

    private static void GetTopLeft(ref int x, ref int y)
    {
        Tile tile = Main.tile[x, y];
        if(tile.frameX > 0)
        {
            x--;
        }
        if(tile.frameY > 18)
        {
            y--;
        }
    }

    public override bool RightClick(int x, int y)
    {
        if(Main.localPlayer.teleportTime == 1f || Main.gamePaused)
        {
            return false;
        }
        Switch(x, y);
        if(Main.netMode != 0)
        {
            BluemagicNet.SendTransceiverSwitch(x, y);
        }
        return true;
    }

    public static void Switch(int x, int y)
    {
        Tile tile = Main.tile[x, y];
        if(tile.frameY <= 18)
        {
            tile.frameY = (short)(18 - tile.frameY);
            string message = tile.frameX == 0 ? "Transmitter: " : "Receiver: ";
            message += tile.frameY == 0 ? "On" : "Off";
            Main.NewText(message, 255, 255, 255, false);
        }
        else
        {
            tile.frameY += 18;
            if(tile.frameY > 18 * 11)
            {
                tile.frameY = 36;
            }
            Main.PlaySound(28, x * 16, y * 16, 0);
        }
    }

    private int GetFrequency(int x, int y)
    {
        int left = Main.tile[x, y + 1].frameY / 18 - 2;
        int right = Main.tile[x + 1, y + 1].frameY / 18 - 2;
        return 10 * left + right;
    }

    public override void HitWire(int x, int y, int wireType)
    {
        if(Main.netMode == 1)
        {
            return;
        }
        GetTopLeft(ref x, ref y);
        if(Transmitting(x, y))
        {
            int frequency = GetFrequency(x, y);
            foreach(Point16 point in positions)
            {
                if(Receiving(point) && frequency == GetFrequency(point.x, point.y))
                {
                    TripWire(point);
                    TripWire(new Point16(point.x + 1, point.y));
                    TripWire(new Point16(point.x, point.y + 1));
                    TripWire(new Point16(point.x + 1, point.y + 1));
                }
            }
        }
    }

    private static bool Transmitting(int x, int y)
    {
        return Main.tile[x, y].frameY == 0;
    }

    private static bool Receiving(Point16 point)
    {
        return Main.tile[point.x + 1, point.y].frameY == 0;
    }

    private static void TripWire(Point16 point)
    {
        byte b;
        if(!Wiring.toProcess.TryGetValue(point, out b))
        {
            Wiring.wireList.PushBack(point);
            Wiring.toProcess.Add(point, 4);
            Wiring.SkipWire(point);
        }
    }

    public override void PostDrawType(SpriteBatch sb, bool solidTiles, int x, int y)
    {
        Tile tile = Main.tile[x, y];
        Vector2 zero = new Vector2((float)Main.offScreenRange, (float)Main.offScreenRange);
        if(Main.drawToScreen)
        {
            zero = Vector2.Zero;
        }
        Main.spriteBatch.Draw(modBase.textures["Tiles/WirelessTransceiverGlow"], new Vector2((float)(x * 16 - (int)Main.screenPosition.X) - 1, (float)(y * 16 - (int)Main.screenPosition.Y) - 1) + zero, new Rectangle?(new Rectangle((int)tile.frameX, (int)tile.frameY, 16, 16)), new Color(200, 200, 200, 0), 0f, default(Vector2), 1f, SpriteEffects.None, 0f);
    }
}}